/* Auspice functions.
 *
 * Auspice Services is copyright (c) 2000-2001 In Mean.
 *     E-mail: <inmean@khmerpop.com>
 *
 * This program is free but copyrighted software; see the file LICENSE for
 * details.
 */

#include "../inc/services.h"
#include "../inc/timeout.h"
#include "../inc/auspice.h"
#include "bs-help.c"

int servbot;
static int nhostline = 0;
int nmessage = 0;
int nhj = 0;
static int hostline_size = 0;
static int message_size = 0;
static int hj_size = 0;

struct message_ {
    char *text;
    short type;
    char *who;
    long reserved[4];
} *lmessages = NULL;

struct hostjoin_ {
    char *host;
    char *chan;
    char *who;
    long reserved[4];
} *hostjoin = NULL;

static struct s_hline {
	char *nick;
	char *host;
	char who[NICKMAX];
	long reserved[4];
} *s_hlines = NULL;

char *botname[64];
char *botmaster[64];

int majdata;

static void do_agenthelp(const char *source);
static void do_agentset(const char *source);

int del_hostline(const char *nick);
void load_hline_db(void);
void save_hline_db(void);
void do_hostline(const char *nick);
void agent(const char *source, char *buf);
void servicesX(const char *source, char *buf);
void servicesW(const char *source, char *buf);
void do_selfdel(const char *source);
int check_ifbot(char *abotname);
int is_servbot(char *nick);
static void abotcom(const char *source, ChannelInfo *ci, char *buf);
void botkick(char *nick, ChannelInfo *ci, char *buf);
void dropmode(char *nick, ChannelInfo *ci);
// static UserData *get_user_data(Channel *c, User *u);

int socket_connected = 0;

/* 1 = connected
   0 = not connected, no failed attempts
   -(x), x = number of failed attempts */

int socket_fd = 0;
long int last_ping;

/*************************** TOOLS *************************************/
char timestamp_buff[50];
char *timestamp(void) {
        struct  tm      tm;
        time_t  timer;

        time(&timer);
        tm = *localtime(&timer); strftime(timestamp_buff, sizeof(timestamp_buff) - 1, "[%H:%M:%S %m/%d/%Y]", &tm);
        return(timestamp_buff);
}

/*************************************************************************/
void load_msg_db(void)
{
    FILE *f;
    int i;

    if (!(f = open_db(s_Agent, MSG_DB, "r")))
        return;

    switch (i = get_file_version(f, MSG_DB)) {
      case 5:
        nmessage = fgetc(f) * 256 + fgetc(f);
        if (nmessage < 8)
            message_size = 16;
        else
            message_size = 2*nmessage;
        lmessages = smalloc(sizeof(*lmessages) * message_size);
        if (!nmessage) {
            fclose(f);
            return;
        }
        if (nmessage != fread(lmessages, sizeof(*lmessages), nmessage, f))
            fatal_perror("Read error on %s", MSG_DB);
        for (i = 0; i < nmessage; i++) {
            lmessages[i].text = read_string(f, MSG_DB);
            lmessages[i].who = read_string(f, MSG_DB);
        }
        break;
      default:
        fatal("Unsupported version (%d) on %s", i, MSG_DB);
    } /* switch (version) */

    close_db(f, MSG_DB);
}
/*************************************************************************/
void save_message(void)
{
    FILE *f;
    int i, count = 0;
    f = open_db(s_Agent, MSG_DB, "w");
    fputc(nmessage/256, f);
    fputc(nmessage & 255, f);
    if (fwrite(lmessages, sizeof(*lmessages), nmessage, f) != nmessage)
        fatal_perror("Write error on %s", MSG_DB);
    for (i = 0; i < nmessage; i++) {
        write_string(lmessages[i].text, f, MSG_DB);
        write_string(lmessages[i].who, f, MSG_DB);
    }
    close_db(f, MSG_DB);
}
/*************************************************************************/
void do_logonmsg(const char *source)
{
	int i;
        for (i = 0; i < nmessage; ++i)
            if (lmessages[i].type == M_LOGON)
                notice(s_GlobalNoticer, source, "%s", lmessages[i].text);
}
/*************************************************************************/
void do_operlogonmsg(const char *source)
{
	int i;
        for (i = 0; i < nmessage; ++i)
            if (lmessages[i].type == M_OPER)
                notice(s_OperServ, source, "%s", lmessages[i].text);
}
/*************************************************************************/
void load_hline_db(void)
{
    FILE *f;
    int i;

    if (!(f = open_db(s_ServicesW, HLINE_DB, "r")))
        return;

    switch (i = get_file_version(f, HLINE_DB)) {
      case 5:
        nhostline = fgetc(f) * 256 + fgetc(f);
        if (nhostline < 8)
            hostline_size = 16;
        else
            hostline_size = 2*nhostline;
        s_hlines = smalloc(sizeof(*s_hlines) * hostline_size);
        if (!nhostline) {
            fclose(f);
            return;
        }
        if (nhostline != fread(s_hlines, sizeof(*s_hlines), nhostline, f))
            fatal_perror("Read error on %s", HLINE_DB);
        for (i = 0; i < nhostline; i++) {
            s_hlines[i].nick = read_string(f, HLINE_DB);
            s_hlines[i].host = read_string(f, HLINE_DB);
        }
        break;
      default:
        fatal("Unsupported version (%d) on %s", i, HLINE_DB);
    } /* switch (version) */

    close_db(f, HLINE_DB);
}
/***********************************************************/
void load_majoin_db(void)
{
    FILE *f;
    int i;

    if (!(f = open_db(s_Agent, HJ_DB, "r")))
        return;

    switch (i = get_file_version(f, HJ_DB)) {
      case 5:
        nhj = fgetc(f) * 256 + fgetc(f);
        if (nhj < 8)
            hj_size = 16;
        else
            hj_size = 2*nhj;
        hostjoin = smalloc(sizeof(*hostjoin) * hj_size);
        if (!nhj) {
            fclose(f);
            return;
        }
        if (nhj != fread(hostjoin, sizeof(*hostjoin), nhj, f))
            fatal_perror("Read error on %s", HJ_DB);
        for (i = 0; i < nhj; i++) {
            hostjoin[i].host = read_string(f, HJ_DB);
            hostjoin[i].chan = read_string(f, HJ_DB);
            hostjoin[i].who = read_string(f, HJ_DB);
        }
        break;
      default:
        fatal("Unsupported version (%d) on %s", i, HJ_DB);
    } /* switch (version) */

    close_db(f, HJ_DB);
}
/**********************************************************/
void save_majoin_db(void)
{
    FILE *f;
    int i, count = 0;
    f = open_db(s_Agent, HJ_DB, "w");
    fputc(nhj/256, f);
    fputc(nhj & 255, f);
    if (fwrite(hostjoin, sizeof(*hostjoin), nhj, f) != nhj)
        fatal_perror("Write error on %s", HJ_DB);
    for (i = 0; i < nhj; i++) {
        write_string(hostjoin[i].host, f, HJ_DB);
        write_string(hostjoin[i].chan, f, HJ_DB);
        write_string(hostjoin[i].who, f, HJ_DB);
    }
    close_db(f, HJ_DB);
}
/***********************************************************/
void load_bot_db(void)
{
    FILE *f;
    char botdata[128];
    *botdata = 0;
    servbot = 1;
    strcat(botdata, "bot.db");
    if (f = fopen(botdata, "r")) {
    while(fgets(botdata, sizeof(botdata), f)) {
	 if (!(botname[servbot])) {
		botname[servbot] = sstrdup(strtok(botdata, "\n"));
	 } else {
		botmaster[servbot] = sstrdup(strtok(botdata, "\n"));

		if (check_ifbot(botname[servbot]))
			servbot++;
	}
    }
    fclose(f);
}
    servbot--;
}
/***********************************************************/
/***********************************************************/
void connectbot(void)
{
    int i;
    if (servbot>0) {
    for (i=1; i<=servbot; i++) {
//  NEWNICK(bi->nick, bi->user, bi->host, bi->real, "+rBS", 1);
//    NEWNICK(botname[i], "ServerBot", services_host, server_name, "+rdqBS", 1);
     send_cmd (NULL, "NICK %s 1 %lu %s %s %s :%s",
    botname[i], time(NULL), "ServerBot", services_host, server_name, "Services Bot");
    send_cmd(botname[i], "MODE %s +qB", botname[i]);
    send_cmd(botname[i], "JOIN %s", snoopchan);
    send_cmd(botname[i],"SWHOIS %s :is a Services Bot",botname[i]);
    } }
}
/***********************************************************/
void save_hline_db(void)
{
    FILE *f;
    int i, count = 0;

    f = open_db(s_ServicesW, HLINE_DB, "w");
    fputc(nhostline/256, f);
    fputc(nhostline & 255, f);
    if (fwrite(s_hlines, sizeof(*s_hlines), nhostline, f) != nhostline)
        fatal_perror("Write error on %s", HLINE_DB);
    for (i = 0; i < nhostline; i++) {
        write_string(s_hlines[i].nick, f, HLINE_DB);
        write_string(s_hlines[i].host, f, HLINE_DB);
    }
    close_db(f, HLINE_DB);

}
/**********************************************************/
void save_bot_db(void)
{
    FILE *f;
    int i;
    f = fopen("bot.db", "w");
    for	(i=1; i<=servbot; i++) {
        fprintf(f, "%s\n", botname[i]);
        fprintf(f, "%s\n", botmaster[i]);
    }
    fclose(f);
}
/**********************************************************/
int is_hostline(const char *nick)
{
    NickInfo *ni;
    int i;

    for (i = 0; i < nhostline; i++) {
        if (stricmp(s_hlines[i].nick,nick) == 0) {
            return 1;
        }
    }
    return 0;
}
/*********************************************************/
void do_autojoin(const char *nick, char *host, char *ident)
{
    int i;
    if ((ident) && (host) && (nhj >0))
    for (i = 0; i < nhj; i++) {
	if (match_wild_nocase(hostjoin[i].host, host)) {
           send_cmd(s_Agent, "SVSJOIN %s %s", nick, hostjoin[i].chan);
        } else if (stricmp(hostjoin[i].host, ident) == 0) {
           send_cmd(s_Agent, "SVSJOIN %s %s", nick, hostjoin[i].chan);
	}
    }
}
/*********************************************************/
int is_botowner(const char *nick, const char *buf)
{
    int i;
    for (i = 1; i <= servbot; i++) {
        if ((stricmp(botname[i],buf) == 0) && (stricmp(botmaster[i],nick)==0)) {
            return 1;
        }
    }
    return 0;
}
/*********************************************************/
void do_hostline(const char *nick)
{
    NickInfo *ni;
    int i;
    for (i = 0; i < nhostline; i++) {
        if (stricmp(s_hlines[i].nick,nick) == 0) {
	    send_cmd(s_Agent,"CHGHOST %s %s",nick,s_hlines[i].host);
	    return;
        }
    }
    return;
}
/**********************************************************/
int del_hostline(const char *nick)
{
    int i;

    for (i = 0; i < nhostline && stricmp(s_hlines[i].nick, nick) != 0; i++);
    if (i < nhostline) {
        free(s_hlines[i].nick);
        free(s_hlines[i].host);
        nhostline--;
        if (i < nhostline)
        bcopy(s_hlines+i+1, s_hlines+i, sizeof(*s_hlines) * (nhostline-i));
        return 1;
    } else {
        return 0;
    }
}
/***********************************************************/
void agent(const char *source, char *buf)
{
    char *cmd;
    char *s, *chan, *nick, *reason, *otherc, *othercc, *othercmd;
    int i;
    User *u = finduser(source);

    cmd = strtok(buf, " ");

   if (!is_services_admin(source)) {
       	notice(s_Agent, source, ERR_ACCESS_DENIED);
       	return;
   }

    if (!cmd) {
        return;
    } else if (stricmp(cmd, "HELP") == 0) {
        do_agenthelp(source);
    } else if (stricmp(cmd, "OP") == 0) {
	chan = strtok(NULL, " ");
	nick = strtok(NULL, "");
	if (!chan) {
		notice(s_Agent, source, "Syntax: OP \2Channel\2 \2Nick\2");
		return;
	}
	if (!nick) {
                if (!is_chanop(source, chan))
		        change_cmode (s_Agent, chan, "+o", source);
		else
			notice(s_Agent, source, "You already an op in %s", chan);
		return;
	}
	else {
                if (!is_chanop(nick, chan))
		        change_cmode (s_Agent, chan, "+o", nick);
		else
			notice(s_Agent, source, "%s already an op in %s", nick, chan);
		return;
	}

    } else if (stricmp(cmd, "NICK") == 0) {
	chan = strtok(NULL, " ");
	nick = strtok(NULL, "");
	if (!chan) {
		notice(s_Agent, source, "Syntax: NICK \2nick\2 \2nick\2");
		return;
	}
	if (!nick) {
		notice(s_Agent, source, "Syntax: NICK \2nick\2 \2nick\2");
		return;
	}
	else {
	        send_cmd (s_Agent, "SVSNICK %s %s 0", chan, nick);
		return;
	}

    } else if (stricmp(cmd, "\1PING") == 0) {
        if (!(s = strtok(NULL, "")))
            s = "\1";
        notice(s_OperServ, source, "\1PING %s", s);
    } else if (stricmp(cmd, "LISTCHANS") == 0) {
	do_aglist(source);
    } else if (stricmp(cmd, "SET") == 0) {
	do_agentset(source);
    } else if (stricmp(cmd, "INFO") == 0) {
        notice(s_Agent, source, "Services Settings Info:");
        notice(s_Agent, source, " ");
        notice(s_Agent, source, "X set mode +oa in Mark/Hold Channel: \2%s\2",(CHANX == 1)?"Enable":"Disable");
        notice(s_Agent, source, "WebTV support, all notice message show in PRIVMSG: \2%s\2", (SPRIV == 1)?"Enable":"Disable");
        notice(s_Agent, source, "Services Database auto update (Risking): \2%s\2", (AUTOUP == 1)?"Enable":"Disable");
        notice(s_Agent, source, "Hiding Services next update info: \2%s\2", (HIDEUME == 1)?"Enable":"Disable");
        notice(s_Agent, source, "Auto kill clone users: \2%s\2", (autokillclone == 1)?"Enable":"Disable");
        notice(s_Agent, source, "Allow user do NS ListChans: \2%s\2", (ALLOWCLIST == 1)?"Enable":"Disable");
        notice(s_Agent, source, "\2%s\2 force user join channel: \2%s\2 on connect.", (AUTOJOINC == 1)?"Enable":"Disbale", ACHANS);
        notice(s_Agent, source, "\2%s\2 force oper join channel: \2%s\2 on oper up.", (OPERJOIN == 1)?"Enable":"Disable", AOCHANS);
        notice(s_Agent, source, "\2%s\2 adding access mask in channel access.", (NS_DACCESS == 0)?"Enable":"Disbale");
        notice(s_Agent, source, "\2%s\2 user set channel JOIN Flag.", (CJOIN == 1)?"Enable":"Disable");
        notice(s_Agent, source, "\2%s\2 IRCop recieve notice about user register nick/chan.", (REGNOTICE == 1)?"Enable":"Disable");
        notice(s_Agent, source, ":End of Services Settings Info");

    } else if (stricmp(cmd, "JOIN") == 0) {
	if (!is_services_root(source)) {
       		notice(s_Agent, source, "Permision Denied.");
	       	return;
	}
	chan = strtok(NULL, " ");
	if (!chan) {
		notice(s_Agent, source, "Syntax: JOIN \2Channel\2");
	} else {
	        send_cmd (s_Agent, "JOIN %s", chan);
        	send_cmd (s_Agent, "MODE %s +a %s", chan, s_Agent);
	}

    } else if (stricmp(cmd, "PART") == 0) {
	if (!is_services_root(source)) {
       		notice(s_Agent, source, "Permision Denied.");
		return;
	}
	chan = strtok(NULL, " ");
	if (!chan) {
		notice(s_Agent, source, "Syntax: PART \2Channel\2");
	} else {
	        send_cmd (s_Agent, "PART %s", chan);
	}
    } else if (stricmp(cmd, "QM") == 0) {
	char *mfrom, *mto, *mmsg;
        char cmdbuf[500];
	char cmdbuf2[500];
        FILE *fp;

	mfrom = strtok(NULL, " ");
	mto = strtok(NULL, " ");
	mmsg = strtok(NULL, "");

	if (!is_services_root(source)) {
       		notice(s_Agent, source, "Permision Denied.");
	       	return;
	}
	if (!mfrom || !mto || !mmsg) {
		notice(s_Agent, source, "Syntax: QM <\2From\2> <\2To\2> <\2Message\2>");
	} else {
      fp = fopen("memo.txt", "a");
        fprintf (fp, "Date: %s\n", timestamp());
        fprintf (fp, "From: %s\n", mfrom);
        fprintf (fp, "To: %s\n", mto);
        fprintf (fp, "Subject: Quick Mail\n");
        fprintf (fp, "%s\n", mmsg);
        fprintf (fp, "\n\n\n");
        fprintf (fp, "----------------------------\n");
        fprintf (fp, "Get Free Web Base EMail from http://mail.pleaselook.com\n");
      fclose (fp);
      sprintf (cmdbuf, "%s -t %s < %s", sendmail_path, mto, "memo.txt");
      sprintf (cmdbuf2, "rm -f %s", "memo.txt");

      system(cmdbuf);
      system(cmdbuf2);
      notice(s_Agent, source, "Quick E-mail sent from %s to %s (%s)", mfrom, mto, mmsg);
	}

    } else if (stricmp(cmd, "KICK") == 0) {
	if (!is_services_root(source)) {
       		notice(s_Agent, source, "Permision Denied.");
		return;
	}
	chan = strtok(NULL, " ");
	nick = strtok(NULL, " ");
	reason = strtok(NULL, "");
	if ((!chan) || (!nick) || (!reason)) {
		notice(s_Agent, source, "Syntax: KICK \2Channel\2 \2nick\2 \2reason\2");
	} else {
	    send_cmd(s_Agent, "KICK %s %s :%s", chan, nick, reason);
	}

    } else if (stricmp(cmd, "EXIT") == 0) {
	if (!is_services_root(source)) {
       		notice(s_Agent, source, "Permision Denied.");
		return;
	}
	nick = strtok(NULL, " ");
	reason = strtok(NULL, "");
	if ((!nick) || (!reason)) {
		notice(s_Agent, source, "Syntax: EXIT \2nick\2 \2reason\2");
	} else {
	    send_cmd(s_Agent, "SVSKILL %s :%s", nick, reason);
	}

    } else if (stricmp(cmd, "HJ") == 0) {
	   if (is_services_admin(source)) {
		othercmd = strtok(NULL, " ");
		otherc = strtok(NULL, " ");
		othercc = strtok(NULL, "");
		if (othercmd) {
		if (stricmp(othercmd, "ADD") == 0) {
			if (!otherc || !othercc)
		      		notice(s_Agent, source, "Syntax: SET HJ  ADD <host> <chan>");
			else {

        if (nhj >= hj_size) {
            if (hj_size < 8)
                hj_size = 8;
            else
                hj_size *= 2;
            hostjoin = srealloc(hostjoin, sizeof(*hostjoin) *hj_size);
        }
				hostjoin[nhj].host = sstrdup(otherc);
				hostjoin[nhj].chan = sstrdup(othercc);
				hostjoin[nhj].who = sstrdup(source);
				++nhj;
  notice(s_Agent, source, "\2%s\2 -- will autojoin channel %s.", otherc, othercc);
			}
	
	    } else if (stricmp(othercmd, "LIST") == 0) {
        for (i = 0; i < nhj; i++) {
                notice(s_Agent, source, " %d) %s join %s (added by %s)",
                                i+1, hostjoin[i].host, hostjoin[i].chan, hostjoin[i].who);
        }
 	 notice(s_Agent, source, "End of List");

	    } else if (stricmp(othercmd, "DEL") == 0) {
		
	int i, j;
       if ((otherc) && (strspn(otherc, "1234567890") == strlen(otherc) &&
                                (i = atoi(otherc)) > 0 && i <=nhj)) {

		for (j=1; j<nhj; ++j) {
			if (j > i) {
				hostjoin[j-1].host = sstrdup(hostjoin[j].host);
				hostjoin[j-1].chan = sstrdup(hostjoin[j].chan);
				hostjoin[j-1].who = sstrdup(hostjoin[j].who);
			} }
		hostjoin[nhj].host = NULL;
		hostjoin[nhj].chan = NULL;
		hostjoin[nhj].who = NULL;
		nhj--;
		notice(s_Agent, source, "Hostjoin #%d is deleted from Autojoin.", i);

	}

	    }
	  } else {
      		notice(s_Agent, source, "Syntax: SET HJ ADD <host> <chan>");
      		notice(s_Agent, source, "Syntax: SET HJ DEL <number>");
          } /* other cmd */
} /* if admin */

/* end HostJoin */

    } else if (stricmp(cmd, "LOGON") == 0) {
	   if (is_services_admin(source)) {
		othercmd = strtok(NULL, " ");
		otherc = strtok(NULL, "");
		if (othercmd) {
		if (stricmp(othercmd, "ADD") == 0) {
			if (!otherc)
		      		notice(s_Agent, source, "Syntax: LOGON ADD <message>");
			else {
        if (nmessage >= message_size) {
            if (message_size < 8)
                message_size = 8;
            else
                message_size *= 2;
            lmessages = srealloc(lmessages, sizeof(*lmessages) *message_size);
        }
				lmessages[nmessage].text = sstrdup(otherc);
				lmessages[nmessage].who = sstrdup(source);
				lmessages[nmessage].type = M_LOGON;
				++nmessage;
	      		notice(s_Agent, source, "\2%s\2 -- added into Logon Message.", otherc);
			}
	
	    } else if (stricmp(othercmd, "LIST") == 0) {
        for (i = 0; i < nmessage; i++) {
		if (lmessages[i].type == M_LOGON)
                notice(s_Agent, source, " %d) %-16s  (added by %s)",
                                i+1, lmessages[i].text, lmessages[i].who);
        }
 	 notice(s_Agent, source, "End of List");

	    } else if (stricmp(othercmd, "DEL") == 0) {
		
	int i, j;
       if ((otherc) && (strspn(otherc, "1234567890") == strlen(otherc) &&
                                (i = atoi(otherc)) > 0 && i <=nmessage)) {

		for (j=1; j<nmessage; ++j) {
			if (j > i) {
				lmessages[j-1].text = sstrdup(lmessages[j].text);
				lmessages[j-1].who = sstrdup(lmessages[j].who);
				lmessages[j-1].type = lmessages[j].type;
			} }
		lmessages[nmessage].text = NULL;
		lmessages[nmessage].who = NULL;
		lmessages[nmessage].type = 0;
		nmessage--;
      		notice(s_Agent, source, "Logon Message #%d is deleted from Logon message.", i);

	}

	    }
	  } else {
      		notice(s_Agent, source, "Syntax: LOGON ADD <message>");
      		notice(s_Agent, source, "Syntax: LOGON LIST");
      		notice(s_Agent, source, "Syntax: LOGON DEL <number>");
          } /* other cmd */
} /* if admin */
/* End Logon */

    } else if (stricmp(cmd, "OPERLOGON") == 0) {
	   if (is_services_admin(source)) {
		othercmd = strtok(NULL, " ");
		otherc = strtok(NULL, "");
		if (othercmd) {
		if (stricmp(othercmd, "ADD") == 0) {
			if (!otherc)
		      		notice(s_Agent, source, "Syntax: OPERLOGON ADD <message>");
			else {
        if (nmessage >= message_size) {
            if (message_size < 8)
                message_size = 8;
            else
                message_size *= 2;
            lmessages = srealloc(lmessages, sizeof(*lmessages) *message_size);
        }
				lmessages[nmessage].text = sstrdup(otherc);
				lmessages[nmessage].who = sstrdup(source);
				lmessages[nmessage].type = M_OPER;
				++nmessage;
	      		notice(s_Agent, source, "\2%s\2 -- added into Oper Logon Message.", otherc);
			}
	
	    } else if (stricmp(othercmd, "LIST") == 0) {
        for (i = 0; i < nmessage; i++) {
		if (lmessages[i].type == M_OPER)
                notice(s_Agent, source, " %d) %-16s  (added by %s)",
                                i+1, lmessages[i].text, lmessages[i].who);
        }
 	 notice(s_Agent, source, "End of List");

	    } else if (stricmp(othercmd, "DEL") == 0) {
		
	int i, j;
       if ((otherc) && (strspn(otherc, "1234567890") == strlen(otherc) &&
                                (i = atoi(otherc)) > 0 && i <=nmessage)) {

		for (j=1; j<nmessage; ++j) {
			if (j > i) {
				lmessages[j-1].text = sstrdup(lmessages[j].text);
				lmessages[j-1].who = sstrdup(lmessages[j].who);
				lmessages[j-1].type = lmessages[j].type;
			} }
		lmessages[nmessage].text = NULL;
		lmessages[nmessage].who = NULL;
		lmessages[nmessage].type = 0;
		nmessage--;
      		notice(s_Agent, source, "Oper Logon Message #%d is deleted from Logon message.", i);

	}

	    }
	  } else {
      		notice(s_Agent, source, "Syntax: OPERLOGON ADD <message>");
      		notice(s_Agent, source, "Syntax: OPERLOGON LIST");
      		notice(s_Agent, source, "Syntax: OPERLOGON DEL <number>");
          } /* other cmd */
} /* if admin */
/* End OperLogon */
    } else {
        notice(s_Agent, source,
                "Unrecognized command \2%s\2.", cmd);
    }
}
/*****************************************************************/
static void do_agenthelp(const char *source)
{
    const char *cmd = strtok(NULL, " ");
    if (!cmd) {
        notice_list(s_Agent, source, agent_help);
    } else if (stricmp(cmd, "SET") == 0) {
        notice_list(s_Agent, source, agent_helpset);
    } else if (stricmp(cmd, "LOGON") == 0) {
      	notice(s_Agent, source, "Syntax: LOGON ADD <message>");
      	notice(s_Agent, source, "Syntax: LOGON LIST");
     	notice(s_Agent, source, "Syntax: LOGON DEL <number>");
    } else {
        notice(s_Agent, source,	"No help available for command \2%s\2.", cmd);
    notice(s_Agent, source, "\2END OF HELP\2");
    }
}
/******************************************************************/
static void do_agentset(const char *source)
{
    const char *cmd = strtok(NULL, " ");
    char *s, *chan, *nick, *reason, *otherc, *reservc, *othercmd, *othercc;
    int i;
    User *u = finduser(source);

    if (!cmd) {
        notice_list(s_Agent, source, agent_helpset);
    } else if (stricmp(cmd, "X") == 0) {
	   if (is_services_coder(source)) {
		otherc = strtok(NULL, " ");
		if (otherc) {
		if (stricmp(otherc, "ON") == 0) {
			CHANX = 1;
	      		notice(s_Agent, source, "X set access in Mark/Hold Channel.");
		} else if (stricmp(otherc, "OFF") == 0) {
			CHANX = 0;
	      		notice(s_Agent, source, "Deactivated X.");
	        } else {
	      		notice(s_Agent, source, "Syntax: SET X ON/OFF");
		} }
   	    }

    } else if ((stricmp(cmd, "SPRIV") == 0) || (stricmp(cmd, "WEBTV") == 0)) {
	   if (is_services_coder(source)) {
		otherc = strtok(NULL, " ");
		if (otherc) {
		if (stricmp(otherc, "ON") == 0) {
			SPRIV = 1;
	      		notice(s_Agent, source, "Service changed from Notice to Private Message.");
		} else if (stricmp(otherc, "OFF") == 0) {
			SPRIV = 0;
	      		notice(s_Agent, source, "Service changed from Private to Notice Message.");
	        } else {
	      		notice(s_Agent, source, "Syntax: SET WEBTV ON/OFF");
		} }
   	    }

    } else if (stricmp(cmd, "AUTOUP") == 0) {
	   if (is_services_coder(source)) {
		otherc = strtok(NULL, " ");
		if (otherc) {
		if (stricmp(otherc, "ON") == 0) {
			AUTOUP = 1;
	      		notice(s_Agent, source, "Service will update database on any change made.");
		} else if (stricmp(otherc, "OFF") == 0) {
			AUTOUP = 0;
	      		notice(s_Agent, source, "Service will update database following the time.");
	        } else {
	      		notice(s_Agent, source, "Syntax: SET AUTOUP ON/OFF");
		} }
   	    }

    } else if (stricmp(cmd, "HIDEUME") == 0) {
	   if (is_services_coder(source)) {
		otherc = strtok(NULL, " ");
		if (otherc) {
		if (stricmp(otherc, "ON") == 0) {
			HIDEUME = 1;
	      		notice(s_Agent, source, "Service next update database info will not show out.");
		} else if (stricmp(otherc, "OFF") == 0) {
			HIDEUME = 0;
	      		notice(s_Agent, source, "Show service next update database info to users.");
	        } else {
	      		notice(s_Agent, source, "Syntax: SET HIDEUME ON/OFF");
		} }
   	    }

    } else if (stricmp(cmd, "KILLCLONE") == 0) {
	   if (is_services_coder(source)) {
		otherc = strtok(NULL, " ");
		if (otherc) {
		if (stricmp(otherc, "ON") == 0) {
			autokillclone = 1;
	      		notice(s_Agent, source, "Enable Auto kill clone.");
		} else if (stricmp(otherc, "OFF") == 0) {
			autokillclone = 0;
	      		notice(s_Agent, source, "Disable Auto kill clone.");
	        } else {
	      		notice(s_Agent, source, "Syntax: SET HIDEUME ON/OFF");
		} }
   	    }

    } else if (stricmp(cmd, "AMASK") == 0) {
	   if (is_services_coder(source)) {
		otherc = strtok(NULL, " ");
		if (otherc) {
		if (stricmp(otherc, "ON") == 0) {
			NS_DACCESS = 1;
	      		notice(s_Agent, source, "Access mask can not be added into channel access list.");
		} else if (stricmp(otherc, "OFF") == 0) {
			NS_DACCESS = 0;
	      		notice(s_Agent, source, "Allow access mask to be added into channel access list.");
	        } else {
	      		notice(s_Agent, source, "Syntax: SET AMASK ON/OFF");
		} }
   	    }

    } else if ((stricmp(cmd, "ALLOWCLIST") == 0) || (stricmp(cmd, "LISTCHANS") == 0)) {
	   if (is_services_coder(source)) {
		otherc = strtok(NULL, " ");
		if (otherc) {
		if (stricmp(otherc, "ON") == 0) {
			ALLOWCLIST = 1;
	      		notice(s_Agent, source, "User is able to use LISTCHANS command.");
		} else if (stricmp(otherc, "OFF") == 0) {
			ALLOWCLIST = 0;
	      		notice(s_Agent, source, "LISTCHANS command are disabled.");
	        } else {
	      		notice(s_Agent, source, "Syntax: SET LISTCHANS ON/OFF");
		} }
   	    }
    } else if (stricmp(cmd, "CJOIN") == 0) {
	   if (is_services_coder(source)) {
		otherc = strtok(NULL, " ");
		if (otherc) {
		if (stricmp(otherc, "ON") == 0) {
			CJOIN = 1;
	      		notice(s_Agent, source, "User now can set ChanServ join channel.");
		} else if (stricmp(otherc, "OFF") == 0) {
			CJOIN = 0;
	      		notice(s_Agent, source, "Disable ChanServ Join option for user.");
	        } else {
	      		notice(s_Agent, source, "Syntax: SET CJOIN ON/OFF");
		} }
   	    }

    } else if (stricmp(cmd, "REGNOTICE") == 0) {
	   if (is_services_coder(source)) {
		otherc = strtok(NULL, " ");
		if (otherc) {
		if (stricmp(otherc, "ON") == 0) {
			REGNOTICE = 1;
	      		notice(s_Agent, source, "Show notice when user registered nick/chan.");
		} else if (stricmp(otherc, "OFF") == 0) {
			REGNOTICE = 0;
	      		notice(s_Agent, source, "Hide user registered nick/chan notice.");
	        } else {
	      		notice(s_Agent, source, "Syntax: SET REGNOTICE ON/OFF");
		} }
   	    }

    } else if (stricmp(cmd, "AUTOJOIN") == 0) {
	   if (is_services_coder(source)) {
		otherc = strtok(NULL, " ");
		if (otherc) {
		if (stricmp(otherc, "ON") == 0) {
			reservc = strtok(NULL, " ");
			if (reservc)	
				strcpy(ACHANS, &*reservc);
			if (!ACHANS) 
				AUTOJOINC = 1;
			else
			if (!(*ACHANS == '#') && (reservc)) {
		      		notice(s_Agent, source, "Syntax: SET AUTOJOIN ON/OFF #<channel>");
			} else {
				AUTOJOINC = 1;
		      		notice(s_Agent, source, "User will auto join channel %s.",ACHANS);
			}
		} else if (stricmp(otherc, "OFF") == 0) {
			AUTOJOINC = 0;
	      		notice(s_Agent, source, "Disable auto join option.");
	        } else {
	      		notice(s_Agent, source, "Syntax: SET AUTOJOIN ON/OFF #<channel>");
		} }
   	    }

    } else if (stricmp(cmd, "OPERJOIN") == 0) {
	   if (is_services_coder(source)) {
		otherc = strtok(NULL, " ");
		if (otherc) {
		if (stricmp(otherc, "ON") == 0) {
			reservc = strtok(NULL, " ");
			if (reservc)
				strcpy(AOCHANS, &*reservc);
			if (!AOCHANS)
				OPERJOIN = 1;
			if (!(*AOCHANS == '#') && (reservc)) {
		      		notice(s_Agent, source, "Syntax: SET OPERJOIN ON/OFF #<channel>");
			} else {
				OPERJOIN = 1;
		      		notice(s_Agent, source, "Oper will auto join channel %s.",AOCHANS);
			}
		} else if (stricmp(otherc, "OFF") == 0) {
			OPERJOIN = 0;
	      		notice(s_Agent, source, "Disable Oper auto join option.");
	        } else {
	      		notice(s_Agent, source, "Syntax: SET OPERJOIN ON/OFF #<channel>");
		} }
   	    }
    } else  {
        notice(s_Agent, source,
                "Unrecognized command \2%s\2.", cmd);
    }
}
/******************************************************************/
void servicesX(const char *source, char *buf)
{
    char *cmd;
    char *s, *chan, *nick, *host;
    int i;
    User *u = finduser(source);

    cmd = strtok(buf, " ");

    if (!cmd) {
        return;
    } else if (stricmp(cmd, "HELP") == 0) {
	if (is_services_oper(source)) {
		notice(s_ServicesX, source, "- MODEQ      Set peace mode in any channel");	
		notice(s_ServicesX, source, "- MODEUQ     Un set peace mode in any channel");	
		notice(s_ServicesX, source, "- HALFOP     Give halfop to current SOP");	
	}
	if (is_services_root(source)) {
		notice(s_ServicesX, source, "- JOIN       Force X join any channel");
		notice(s_ServicesX, source, "- PART       Force X part any channel");
	}
    } else if (stricmp(cmd, "\1PING") == 0) {
        if (!(s = strtok(NULL, "")))
            s = "\1";
        notice(s_OperServ, source, "\1PING %s", s);

    } else if (stricmp(cmd, "JOIN") == 0) {
	if (!is_services_root(source)) {
       		notice(s_ServicesX, source, "Permision Denied.");
	       	return;
	}
	chan = strtok(NULL, " ");
	if (!chan) {
		notice(s_ServicesX, source, "Syntax: JOIN \2Channel\2");
	} else {
	        send_cmd (s_ServicesX, "JOIN %s", chan);
        	send_cmd (s_ServicesX, "MODE %s +a %s", chan, s_ServicesX);
	}

    } else if (stricmp(cmd, "PART") == 0) {
	if (!is_services_root(source)) {
       		notice(s_ServicesX, source, "Permision Denied.");
	}
	chan = strtok(NULL, " ");
	if (!chan) {
		notice(s_ServicesX, source, "Syntax: PART \2Channel\2");
	} else {
	        send_cmd (s_ServicesX, "PART %s", chan);
	}

    } else if (stricmp(cmd, "MODEQ") == 0) {
	if (!is_services_oper(source)) {
       		notice(s_ServicesX, source, "Permision Denied.");
	}
	chan = strtok(NULL, " ");
	if (!chan) {
		notice(s_ServicesX, source, "Syntax: MODEQ \2Channel\2");
	} else {
	        send_cmd (s_ServicesX, "MODE %s +Q", chan);
		notice(s_ServicesX, chan, "Request +Q by oper: %s", source);
	        slog("OPER Q %s (%s!%s@%s) %s", u->nick, u->nick, u->username, u->host, chan);
	}
    } else if (stricmp(cmd, "MODEUQ") == 0) {
	if (!is_services_oper(source)) {
       		notice(s_ServicesX, source, "Permision Denied.");
	}
	chan = strtok(NULL, " ");
	if (!chan) {
		notice(s_ServicesX, source, "Syntax: MODEUQ \2Channel\2");
	} else {
	        send_cmd (s_ServicesX, "MODE %s -Q", chan);
		notice(s_ServicesX, chan, "Request -Q by oper: %s", source);
	        slog("OPER -Q %s (%s!%s@%s) %s", u->nick, u->nick, u->username, u->host, chan);
	}
    } else if (stricmp(cmd, "HALFOP") == 0) {
	chan = strtok(NULL, " ");
	if (!chan) {
		notice(s_ServicesX, source, "Syntax: HALFOP \2Channel\2");
		return;
	} else {
                if (!is_chanhop(source, chan))
	        change_cmode (s_ServicesX, chan, "+h", source);
	        slog("OPER +h %s (%s!%s@%s) %s", u->nick, u->nick, u->username, u->host, chan);
		return;
	}
    } else {
        notice(s_ServicesX, source,
                "Unrecognized command \2%s\2.", cmd);
    }
}

/***************************************************************/
void servicesW(const char *source, char *buf)
{
    char *cmd;
    char *s, *chan, *nick, *host, *info;
    int i, j;
    User *u = finduser(source);
    NickInfo *ni;

    cmd = strtok(buf, " ");

    if (!cmd) {
	return;
    } else if (stricmp(cmd, "HELP") == 0) {
	if (is_services_root(source)) {
		notice(s_ServicesW, source, "- JOIN       Force W join any channel");
		notice(s_ServicesW, source, "- PART       Force W part any channel");
		notice(s_ServicesW, source, "- SPY        Check all action taken by current host.");
	}
    } else if (stricmp(cmd, "\1PING") == 0) {
        if (!(s = strtok(NULL, "")))
            s = "\1";
        notice(s_OperServ, source, "\1PING %s", s);

    } else if (stricmp(cmd, "JOIN") == 0) {

	if (!is_services_root(source)) {
       		notice(s_ServicesW, source, "Permision Denied.");
	       	return;
	}
	chan = strtok(NULL, " ");
	if (!chan) {
		notice(s_ServicesW, source, "Syntax: JOIN \2Channel\2");
	} else {
	        send_cmd (s_ServicesW, "JOIN %s", chan);
        	send_cmd (s_ServicesW, "MODE %s +a %s", chan, s_ServicesW);
	}

    } else if (stricmp(cmd, "PART") == 0) {

	if (!is_services_root(source)) {
       		notice(s_ServicesW, source, "Permision Denied.");
	       	return;
	}
	chan = strtok(NULL, " ");
	if (!chan) {
		notice(s_ServicesW, source, "Syntax: PART \2Channel\2");
	} else {
	        send_cmd (s_ServicesW, "PART %s", chan);
	}

    } else if (stricmp(cmd, "SPY") == 0) {
	if (!is_services_root(source)) {
       		notice(s_ServicesW, source, "Permision Denied.");
	}
	spyhost = strtok(NULL, " ");
	if (!spyhost) {
		notice(s_ServicesW, source, "Syntax: SPYHOST \2<Host>\2");
//		strcpy(spyhost, NULL);
	} else {
		notice(s_ServicesX, source, "All action taken by host: \2%s\2 will show in #debug.", spyhost);
	}

   }
}
/***********************************************************************/
void do_selfdel(const char *source)
{
    ChannelInfo *ci;
    char *chan = strtok(NULL, " ");
    User *u = finduser(source);
    int acc = 0;
    int i, f=0, c=0, z;
    ChanAccess *access;

    if (!chan) {
        notice(s_NickServ, source, "Syntax: \2SELFDEL\2 <#channel>");
        notice(s_NickServ, source,
              "\2/MSG %s HELP SELFDEL\2 fo more information.",
	      s_NickServ);
	return;
    }

    if (readonly) {
	notice(s_ChanServ, source,
	    "Warning: Services is in read-only mode.  Changes will not be saved.");
    }

    if (!(ci = cs_findchan(chan))) {
        notice(s_NickServ, source,"Channel %s not registered", chan);
	    return;
    }
    acc = get_access(u, ci);

    if (acc == 3) {
            for (access = ci->access, z = 0; z < ci->accesscount;
                                                        ++access, ++z) {
                if ((stricmp(access->name, source) == 0)
                        && (access->level == 3))
                    break;
            }
            free(access->name);
            access->in_use = 0;
            access->name = NULL;
              --ci->accesscount;
              if (z < ci->accesscount)
                  bcopy(access+1, access, sizeof(ChanAccess)
                                           * (ci->accesscount-z));
    notice(s_ChanServ, source,	"You have been deleted from VOP in \2%s\2.", ci->name);
    } else if (acc == 4) {
            for (access = ci->access, z = 0; z < ci->accesscount;
                                                        ++access, ++z) {
                if ((stricmp(access->name, source) == 0)
                        && (access->level == 4))
                    break;
            }
            free(access->name);
            access->in_use = 0;
            access->name = NULL;
              --ci->accesscount;
              if (z < ci->accesscount)
                  bcopy(access+1, access, sizeof(ChanAccess)
                                           * (ci->accesscount-z));
    notice(s_ChanServ, source,	"You have been deleted from HOP in \2%s\2.",  ci->name);

    } else if (acc == 5) {
            for (access = ci->access, z = 0; z < ci->accesscount;
                                                        ++access, ++z) {
                if ((stricmp(access->name, source) == 0)
                        && (access->level == 5))
                    break;
            }
            free(access->name);
            access->in_use = 0;
            access->name = NULL;
              --ci->accesscount;
              if (z < ci->accesscount)
                  bcopy(access+1, access, sizeof(ChanAccess)
                                           * (ci->accesscount-z));
    notice(s_ChanServ, source,	"You have been deleted from AOP in \2%s\2.", ci->name);

    } else if (acc == 10) {
            for (access = ci->access, z = 0; z < ci->accesscount;
                                                        ++access, ++z) {
                if ((stricmp(access->name, source) == 0)
                        && (access->level == 10))
                    break;
            }
            free(access->name);
            access->in_use = 0;
            access->name = NULL;
              --ci->accesscount;
              if (z < ci->accesscount)
                  bcopy(access+1, access, sizeof(ChanAccess)
                                           * (ci->accesscount-z));
    notice(s_ChanServ, source,	"You have been deleted from SOP in \2%s\2.", ci->name);

    } else if (acc == 13) {
            for (access = ci->access, z = 0; z < ci->accesscount;
                                                        ++access, ++z) {
                if ((stricmp(access->name, source) == 0)
                        && (access->level == 13))
                    break;
            }
            free(access->name);
            access->in_use = 0;
            access->name = NULL;
              --ci->accesscount;
              if (z < ci->accesscount)
                  bcopy(access+1, access, sizeof(ChanAccess)
                                           * (ci->accesscount-z));
    notice(s_ChanServ, source,	"You have been deleted from CF in \2%s\2.", ci->name);

    } else if (acc == 15) {
        notice(s_NickServ, source,"This option can't use with Founder", ci->name);
    } else {
        notice(s_NickServ, source,"You have no access in \37%s\37", ci->name);
    }	
    show_u_next_db(u, s_ChanServ);
}
/**********************************************************************/
void SpamCheck(const char *source, char *buf)
{
    char *cmd, *s, *chan;
    char orig[BUFSIZE];
    strscpy (orig, buf, sizeof (orig));
    cmd = strtok(buf, " ");

    if (cmd) 
	    if (stricmp (cmd, "\1PING") == 0)
	    {
	        if (!(s = strtok (NULL, "")))
	            s = "\1";
	        notice(s_OperServ, source, "\1PING %s", s);

	    } else if (stricmp (cmd, "\1VERSION\1") == 0) {
	        if (!(s = strtok (NULL, "")))
        	    s = "\1";
	        notice(s_SpamChecker, source, "\1VERSION ircII-2.8.2-EPIC4pre1.001");

	    } else if (stricmp(cmd, "JOIN") == 0) {

		if (!is_services_root(source)) {
		       	return;
		}
		chan = strtok(NULL, " ");
		if (!chan) {
			notice(s_SpamChecker, source, "Syntax: JOIN \2Channel\2");
		} else {
		        send_cmd (s_SpamChecker, "JOIN %s", chan);
		}

	    } else if (stricmp(cmd, "PART") == 0) {

		if (!is_services_root(source)) {
		       	return;
		}
		chan = strtok(NULL, " ");
		if (!chan) {
			notice(s_SpamChecker, source, "Syntax: PART  \2Channel\2");
		} else {
		        send_cmd (s_SpamChecker, "PART %s", chan);
		}

	    } else
	       if (!is_services_helpop(source))
		       wallops(s_OperServ, "Possible spam from \2%s\2: %s", source, orig);
}
/*********************************************************************/
int check_password(const char *plaintext, const char *password)
{
    if (strcmp(plaintext, password) == 0)
	return 1;
    else
        return 0;
}
/*******************************************************************/
int check_ifbot(char *abotname)
{
	int i;
	for (i=1; i<=servbot; i++)
		if (stricmp(abotname, botname[i]) == 0)
			return 1;
	return 0;
}
/*******************************************************************/
void botjoin(char *chan)
{
	ChannelInfo *ci;
	ci = cs_findchan(chan);
	if (ci && ci->bot) {
	        send_cmd(ci->bot, "JOIN %s", ci->name);	
		send_cmd(ci->bot, "MODE %s +o %s", chan, ci->bot);
	}
}
/*******************************************************************/
void botpart(char *chan)
{
	ChannelInfo *ci;
	ci = cs_findchan(chan);
	if (ci && ci->bot && !(ci->botflag & CBI_BOTSTAY))
	        send_cmd(ci->bot, "PART %s", ci->name);	
}
/*******************************************************************/
void botserv(const char *source, const char *thebot, char *buf)
{
    char *cmd, *s, *chan, *bmsg;
    int i;
    char orig[BUFSIZE];
    NickInfo *ni;
    strscpy (orig, buf, sizeof (orig));
    cmd = strtok(buf, " ");

    if (cmd)
            if (stricmp (cmd, "\1PING") == 0) {
                if (!(s = strtok (NULL, "")))
                    s = "\1";
                notice(thebot, source, "\1PING %s", s);

            } else if (stricmp (cmd, "\1VERSION\1") == 0) {
                if (!(s = strtok (NULL, "")))
                    s = "\1";
             notice(thebot, source, "%sVERSION Auspice IRC Services %s", s, s);
	
           } else if (!is_services_admin(source)) {
        	chan = strtok(NULL, "");
	    	for (i = 1; (i <= servbot) && (stricmp(botname[i],thebot) != 0); i++)
		;
            
	    	ni = findnick(botmaster[i]);

	    	if (ni && finduser(botmaster[i]))
            	if ((ni->flags & NI_IDENTIFIED) && (ni->flags & NI_BOTMSG)) {
	        	send_cmd(thebot, "PRIVMSG %s :(%s) %s %s", ni->nick, source, cmd, (chan)?chan:"");
            	}
	    	return;
          
            } else if (stricmp(cmd, "JOIN") == 0) {
                chan = strtok(NULL, " ");
                if (!chan) {
                        notice(thebot, source, "Syntax: JOIN \2Channel\2");
                } else {
			if (is_botowner(source, thebot) || is_services_root(source))
	                        send_cmd (thebot, "JOIN %s", chan);
			else
	                        notice(thebot, source, "Sorry you are not my master.");		
                }
            } else if (stricmp(cmd, "HELP") == 0) {
		if (is_botowner(source, thebot) || is_services_root(source)) {
		notice(thebot, source, "- JOIN      Join channel");	
		notice(thebot, source, "- PART      Part channel");	
		notice(thebot, source, "- SAY       Say in channel");	
		notice(thebot, source, "- ACT       Act in channel");	
		notice(thebot, source, "- QUIT      Quit the Bot");	
	        notice(thebot, source, "\2ENF OF HELP\2");
		} else
	                notice(thebot, source, "Sorry you are not my master.");		
 

            } else if (stricmp(cmd, "SAY") == 0) {
                chan = strtok(NULL, " ");
		bmsg = strtok(NULL, "");
                if (!chan || !bmsg) {
                        notice(thebot, source, "Syntax: SAY \2Channel\2 \2Message\2");
                } else {
			if (is_botowner(source, thebot) || is_services_root(source))
			     if (finduser(chan) || findchan(chan))
			     	send_cmd (thebot, "PRIVMSG %s :%s", chan, bmsg);
			     else
	                        notice(thebot, source, "Can not send to non exist channel or any services bots.");					
			else
	                        notice(thebot, source, "Sorry you are not my master.");		
                }

            } else if (stricmp(cmd, "ACT") == 0) {
                chan = strtok(NULL, " ");
		bmsg = strtok(NULL, "");
                if (!chan || !bmsg) {
                        notice(thebot, source, "Syntax: ACT \2Channel\2 \2Message\2");
                } else {
			if (is_botowner(source, thebot) || is_services_root(source))
			     if (finduser(chan) || findchan(chan))
			     send_cmd (thebot, "PRIVMSG %s :ACTION %s",chan, bmsg);
			     else
	                        notice(thebot, source, "Can not send to non exist channel or any services bots.");					

			else
	                        notice(thebot, source, "Sorry you are not my master.");		
                }

            } else if (stricmp(cmd, "PART") == 0) {
	        ChannelInfo *ci;
                chan = strtok(NULL, " ");
		ci = cs_findchan(chan);
                if (!chan) {
                        notice(thebot, source, "Syntax: PART \2Channel\2");
		} else if (ci && (stricmp(ci->bot,thebot)==0)) {
                        notice(thebot, source, "Sorry, this bot have been assigned for this channel already.");
                } else {
			if (is_botowner(source, thebot) || is_services_root(source))
	                        send_cmd (thebot, "PART %s", chan);
			else
	                        notice(thebot, source, "Sorry you are not my master.");		
                }
            } else if (stricmp(cmd, "QUIT") == 0) {
		if (is_botowner(source, thebot) || is_services_root(source)) {
				int i, j;
			        do_expireabot(thebot);
		                send_cmd (thebot, "QUIT :Quit by %s", source);
			        wallops(s_BotServ, "Services bot \2%s\2 quited by \2%s\2", thebot, source);
				for (i=1; i<=servbot; i++)
				if (stricmp(botname[i], thebot) == 0)
					break;

				for (j=i; j<=(servbot-1) ; ++j) {
					strcpy(botname[j], botname[j+1]);
					strcpy(botmaster[j], botmaster[j+1]);
				}
					botname[servbot] = NULL;
					botmaster[servbot] = NULL;
					--servbot;


			} else
	                        notice(thebot, source, "Sorry you are not my master.");		
            }
}
/*****************************************************************/
void statserv(const char *source, char *buf)
{
    char *cmd, *s, *chan;
    char orig[BUFSIZE];
    strscpy (orig, buf, sizeof (orig));
    cmd = strtok(buf, " ");

    if (cmd) 
	    if (stricmp (cmd, "\1PING") == 0)
	    {
	        if (!(s = strtok (NULL, "")))
	            s = "\1";
	        notice(s_OperServ, source, "\1PING %s", s);

	    } else if (stricmp (cmd, "\1VERSION\1") == 0) {
	        if (!(s = strtok (NULL, "")))
        	    s = "\1";
	        notice(s_StatServ, source, "%sVERSION Auspice IRC Services %s", s, s);

	    } else if (stricmp(cmd, "JOIN") == 0) {

		if (!is_services_root(source)) {
		       	return;
		}
		chan = strtok(NULL, " ");
		if (!chan) {
			notice(s_StatServ, source, "Syntax: JOIN \2Channel\2");
		} else {
		        send_cmd (s_StatServ, "JOIN %s", chan);
		}

	    } else if (stricmp(cmd, "PART") == 0) {

		if (!is_services_root(source)) {
		       	return;
		}
		chan = strtok(NULL, " ");
		if (!chan) {
			notice(s_StatServ, source, "Syntax: PART \2Channel\2");
		} else {
		        send_cmd (s_StatServ, "PART %s", chan);
		}

	    } else if (stricmp(cmd, "STATS") == 0) {
		do_mem(source);
	    } else if (stricmp(cmd, "SERVER") == 0) {
		do_statss(source);
	    } else if (stricmp(cmd, "SERVERS") == 0) {
		do_stats_server(source);
	    } else if (stricmp(cmd, "HELP") == 0) {
		notice(s_StatServ, source,"- SERVERS    List all servers.");
		notice(s_StatServ, source,"- STATS      Show Services Db Record.");
		if (is_services_oper(source)) {
		notice(s_StatServ, source,"- SERVER      List all servers with desc.");	
		}
		if (is_services_root(source)) {
		notice(s_StatServ, source,"- JOIN       Force StatServ join any channel.");	
		notice(s_StatServ, source,"- PART       Force StatServ leave any channel.");	
		}
	    }
}
/***********************************************************************/
int is_servbot(char *nick)
{
      if ((stricmp(nick, s_OperServ) == 0)
	   || (stricmp(nick, s_RootServ) == 0)
	   || (stricmp(nick, s_NickServ) == 0)
	   || (stricmp(nick, s_MemoServ) == 0)
	   || (stricmp(nick, s_ChanServ) == 0)
	   || (stricmp(nick, s_HelpServ) == 0)
	   || (stricmp(nick, s_Agent) == 0)
	   || (stricmp(nick, s_ServicesX) == 0)
	   || (stricmp(nick, s_HostServ) == 0)
	   || (stricmp(nick, s_MassServ) == 0)
	   || (stricmp(nick, s_BotServ) == 0)
	   || (stricmp(nick, s_ServicesW) == 0)
	   || (stricmp(nick, s_StatServ) == 0)
	   || (stricmp(nick, s_SpamChecker) == 0)
	   || (check_ifbot(nick) == 1))
		return 1;
	return 0;
		
}
/***********************************************************************/
void HostServ(const char *source, char *buf)
{
    char *cmd;
    char *s, *chan, *nick, *host, *info;
    int i, j;
    User *u = finduser(source);
    NickInfo *ni;

    cmd = strtok(buf, " ");

    if (!cmd) {
	return;
    } else if (stricmp(cmd, "HELP") == 0) {
	if (is_services_helpop(source))
		notice(s_HostServ, source, "- SETHOST   Change you hostname");
	if (is_services_oper(source)) {
		notice(s_HostServ, source, "- ADD       Add auto sethost to a nick.");	
		notice(s_HostServ, source, "- DEL       Delete auto sethost from a nick.");	
		notice(s_HostServ, source, "- LIST      show nick list of auto sethost.");	
		notice(s_HostServ, source, "- CHANGE    Modify nickname host.");
	}
    } else if (stricmp(cmd, "\1PING") == 0) {
        if (!(s = strtok(NULL, "")))
            s = "\1";
        notice(s_OperServ, source, "\1PING %s", s);

    } else if (stricmp(cmd, "JOIN") == 0) {

	if (!is_services_root(source)) {
       		notice(s_HostServ, source, "Permision Denied.");
	       	return;
	}
	chan = strtok(NULL, " ");
	if (!chan) {
		notice(s_HostServ, source, "Syntax: JOIN \2Channel\2");
	} else {
	        send_cmd (s_HostServ, "JOIN %s", chan);
        	send_cmd (s_HostServ, "MODE %s +a %s", chan, s_ServicesW);
	}

    } else if (stricmp(cmd, "PART") == 0) {

	if (!is_services_root(source)) {
       		notice(s_HostServ, source, "Permision Denied.");
	       	return;
	}
	chan = strtok(NULL, " ");
	if (!chan) {
		notice(s_HostServ, source, "Syntax: PART \2Channel\2");
	} else {
	        send_cmd (s_HostServ, "PART %s", chan);
	}

   } else if (stricmp(cmd, "LIST") == 0) {
        s = strtok(NULL, " ");
	if (!is_services_oper(source)) {
       		notice(s_HostServ, source, "Permision Denied.");
	       	return;
	}
        if (!s)
            s = "*";
        slog("HO L %s (%s!%s@%s)", u->nick, u->nick, u->username, u->host);
        notice(s_HostServ, source, "\2Current Host list:\2");
        for (i = 0; i < nhostline; i++) {
            if (!s || match_wild(s, s_hlines[i].nick)) {

              if (ni = findnick(s_hlines[i].nick))
                notice(s_HostServ, source, " %d) %-16s (%s) added by %s",
                                i+1, s_hlines[i].nick, s_hlines[i].host, s_hlines[i].who);
              else
                notice(s_HostServ, source, " %d) %-16s (Not Registered)",
                                i+1, s_hlines[i].nick);
            }
        }
        notice(s_HostServ,source,"\2End of List\2");

    	} else if (stricmp(cmd,"ADD") == 0) {
        nick = strtok(NULL, " ");
        host = strtok(NULL, "");
        if (!nick || !host) {
            notice(s_HostServ, source, "Syntax: \2ADD\2 <nick> <host>");
            return;
        }
        if (!findnick(nick)) {
                notice(s_HostServ, source, "Nick %s isn't registered!", nick);
                return;
        }

	if (!is_services_oper(source)) {
       		notice(s_HostServ, source, "Permision Denied.");
	       	return;
	}

        for (i = 0; i < nhostline; i++) {
            if (stricmp(s_hlines[i].nick,nick) == 0) {
                notice(s_HostServ,source,"%s is already present "
                    "on the Host list.", nick);
                return;
            }
        }

        if (nhostline >= hostline_size) {
            if (hostline_size < 8)
                hostline_size = 8;
            else
                hostline_size *= 2;
            s_hlines = srealloc(s_hlines, sizeof(*s_hlines) *hostline_size);
        }

        if (!nhostline || (nhostline < MAX_HOSTSERV) || (MAX_HOSTSERV == 0)) {
            s_hlines[nhostline].nick = sstrdup(nick);
            s_hlines[nhostline].host = sstrdup(host);
            strscpy(s_hlines[nhostline].who, source, NICKMAX);

            notice(s_HostServ, source, "Added nick %s into auto change host list (%s).",nick,host);
	    show_next_db(source, s_ServicesW);
            slog("HO A %s %s by %s", nick, host, source);
            ++nhostline;
            if (readonly) {
                notice(s_HostServ, source,
                    "\2Notice:\2 Changes will not be saved! "
                    " Services is in read-only mode.");
            }
        }
        else {
            notice(s_HostServ,source,"Sorry, Host list is full.");
            wallops(SERVER_NAME,"\2Warning\2: host(nick) list is full. (%d)", MAX_HOSTSERV);
        }

    } else if (stricmp(cmd, "CHANGE") == 0) {
        int i;
        nick = strtok(NULL, " ");
        info = strtok(NULL, "");

        if (!is_services_oper(source)) {
            notice(s_HostServ, source, "Permission denied.");
            return;
        }

        if ((nick == NULL) || (info == NULL)) {
          notice(s_HostServ, source, "Syntax: \2CHANGE\2 <nick/number> <newhost>");
          return;
	}
            if (strspn(nick, "1234567890") == strlen(nick) &&
                                (i = atoi(nick)) > 0 && i <= nhostline) {

                free(s_hlines[i-1].host);
                s_hlines[i-1].host = sstrdup(info);
	        strscpy(s_hlines[i-1].who, source, NICKMAX);
                notice(s_HostServ, source, "Nick %s auto change host to %s",
                             s_hlines[i-1].nick, sstrdup(info));
                return;
            }

         for (i = 0; i < nhostline; i++) {
               if (stricmp(s_hlines[i].nick, nick) == 0) {

                   free(s_hlines[i].host);
                   s_hlines[i].host = sstrdup(info);
	           strscpy(s_hlines[i].who, source, NICKMAX);
	           notice(s_HostServ, source, "Nick %s auto change host to %s",
                             s_hlines[i].nick, sstrdup(info));
               }
         }

    } else if (stricmp(cmd, "DEL") == 0) {
        int i;
	if (!is_services_oper(source)) {
       		notice(s_HostServ, source, "Permision Denied.");
	       	return;
	}
        if (nick = strtok(NULL, " ")) {
            if (strspn(nick, "1234567890") == strlen(nick) &&
                                (i = atoi(nick)) > 0 && i <= nhostline) {
                strcpy(nick,s_hlines[i-1].nick);
            }
            if (del_hostline(nick)) {
                notice(s_HostServ, source, "%s removed from Host list.", nick);
                slog("HO D %s (%s!%s@%s)",
                nick, u->nick, u->username, u->host);

                if (readonly) {
                    notice(s_HostServ, source,
                        "\2Notice:\2 Changes will not be saved!  Services is in readonly mode.");
                }
            } else {
                notice(s_HostServ, source, "%s not found on Host list.",
                    nick);
            }
        } else {
            notice(s_HostServ, source, "Syntax: \2DEL\2 <nick>");
        }

    } else if (stricmp(cmd, "SETHOST") == 0) {
	if (!is_services_helpop(source)) {
       		notice(s_HostServ, source, "Permision Denied.");
	       	return;
	}
	chan = strtok(NULL, " ");  /* Chan = Host */
	if (!chan) {
		notice(s_HostServ, source, "Syntax: SETHOST \2<host>\2");
	} else {
	        send_cmd (s_HostServ, "CHGHOST %s %s", source, chan);
		notice(s_HostServ, source, "Your host name set to \2%s\2", chan);
		notice(s_HostServ, source, "To turn it off type /mode %s -x", source);
	        slog("OPER HOST %s (%s!%s@%s) %s", u->nick, u->nick, u->username, u->host, chan);
	}
   }
}
/***********************************************************************/
void massserv(const char *source, char *buf)
{
    char *cmd, *s, *chan;
    char orig[BUFSIZE];
    strscpy (orig, buf, sizeof (orig));
    cmd = strtok(buf, " ");

    if (cmd) 
	    if (stricmp (cmd, "\1PING") == 0)
	    {
	        if (!(s = strtok (NULL, "")))
	            s = "\1";
	        notice(s_MassServ, source, "\1PING %s", s);

	    } else if (stricmp (cmd, "\1VERSION\1") == 0) {
	        if (!(s = strtok (NULL, "")))
        	    s = "\1";
	        notice(s_MassServ, source, "%sVERSION Auspice IRC Services %s", s, s);

	    } else if (!is_services_root(source)) {
		notice(s_MassServ, source, "Permission denied.");
	       	return;
	    } else if (stricmp(cmd, "JOIN") == 0) {

		if (!is_services_root(source)) {
			notice(s_MassServ, source, "Permission denied.");
		       	return;
		}
		chan = strtok(NULL, " ");
		if (!chan) {
			notice(s_MassServ, source, "Syntax: JOIN \2Channel\2");
		} else {
		        send_cmd (s_MassServ, "JOIN %s", chan);
		}

	    } else if (stricmp(cmd, "PART") == 0) {

		if (!is_services_root(source)) {
			notice(s_MassServ, source, "Permission denied.");
		       	return;
		}
		chan = strtok(NULL, " ");
		if (!chan) {
			notice(s_MassServ, source, "Syntax: PART \2Channel\2");
		} else {
		        send_cmd (s_MassServ, "PART %s", chan);
		}

            } else if (stricmp(cmd, "SERVJOIN") == 0) {
	        chan = strtok(NULL, " ");
		if (!is_services_root(source)) {
			notice(s_MassServ, source, "Permission denied.");
		       	return;
		}
	        if (!chan) {
		    notice(s_MassServ, source, "Syntax: SERVJOIN <channel>");
	            return;
		}
		send_cmd(s_OperServ, "JOIN %s", chan);
		send_cmd(s_OperServ, "MODE %s +o %s", chan, s_OperServ);
		send_cmd(s_RootServ, "JOIN %s", chan);
	        send_cmd(s_RootServ, "MODE %s +o %s", chan, s_RootServ);
	        send_cmd(s_NickServ, "JOIN %s", chan);
	        send_cmd(s_NickServ, "MODE %s +o %s", chan, s_NickServ);
	        send_cmd(s_ChanServ, "JOIN %s", chan);
	        send_cmd(s_ChanServ, "MODE %s +o %s", chan, s_ChanServ);
	        send_cmd(s_MemoServ, "JOIN %s", chan);
	        send_cmd(s_MemoServ, "MODE %s +o %s", chan, s_MemoServ);
	        send_cmd(s_HelpServ, "JOIN %s", chan);
	        send_cmd(s_HelpServ, "MODE %s +o %s", chan, s_HelpServ);
	        send_cmd(s_Agent, "JOIN %s", chan);
	        send_cmd(s_Agent, "MODE %s +o %s", chan, s_Agent);
	        send_cmd(s_ServicesX, "JOIN %s", chan);
	        send_cmd(s_ServicesX, "MODE %s +o %s", chan, s_ServicesX);
	        send_cmd(s_ServicesW, "JOIN %s", chan);
	        send_cmd(s_ServicesW, "MODE %s +o %s", chan, s_ServicesW);
	        send_cmd(s_GlobalNoticer, "JOIN %s", chan);
	        send_cmd(s_GlobalNoticer, "MODE %s +o %s", chan, s_GlobalNoticer);
	        send_cmd(s_SpamChecker, "JOIN %s", chan);
	        send_cmd(s_SpamChecker, "MODE %s +o %s", chan, s_SpamChecker);
	        send_cmd(s_StatServ, "JOIN %s", chan);
	        send_cmd(s_StatServ, "MODE %s +o %s", chan, s_StatServ);
	        send_cmd(s_MassServ, "JOIN %s", chan);
	        send_cmd(s_MassServ, "MODE %s +o %s", chan, s_MassServ);
	        send_cmd(s_BotServ, "JOIN %s", chan);
	        send_cmd(s_BotServ, "MODE %s +o %s", chan, s_BotServ);
	        send_cmd(s_HostServ, "JOIN %s", chan);
	        send_cmd(s_HostServ, "MODE %s +o %s", chan, s_HostServ);
            } else if (stricmp(cmd, "SERVPART") == 0) {
	        chan = strtok(NULL, " ");
		if (!is_services_root(source)) {
			notice(s_MassServ, source, "Permission denied.");
		       	return;
		}
	        if (!chan) {
		    notice(s_MassServ, source, "Syntax: SERVPART <channel>");
	            return;
		}
	        send_cmd(s_OperServ, "PART %s", chan);
	        send_cmd(s_RootServ, "PART %s", chan);
	        send_cmd(s_NickServ, "PART %s", chan);
	        send_cmd(s_ChanServ, "PART %s", chan);
	        send_cmd(s_MemoServ, "PART %s", chan);
	        send_cmd(s_HelpServ, "PART %s", chan);
	        send_cmd(s_Agent, "PART %s", chan);
	        send_cmd(s_ServicesX, "PART %s", chan);
	        send_cmd(s_ServicesW, "PART %s", chan);
	        send_cmd(s_GlobalNoticer, "PART %s", chan);
	        send_cmd(s_SpamChecker, "PART %s", chan);
	        send_cmd(s_StatServ, "PART %s", chan);
	        send_cmd(s_BotServ, "PART %s", chan);
	        send_cmd(s_MassServ, "PART %s", chan);
	        send_cmd(s_HostServ, "PART %s", chan);

	    } else if (stricmp(cmd, "ALLBOTJOIN") == 0 ) {
		int i;
	        chan = strtok(NULL, " ");
		if (!is_services_root(source)) {
			notice(s_MassServ, source, "Permission denied.");
		       	return;
		}
	        if (!chan) {
		    notice(s_MassServ, source, "Syntax: ALLBOTJOIN <channel>");
	            return;
		}
		for (i=1; i<=servbot; i++) {
		        send_cmd(botname[i], "JOIN %s", chan);
			send_cmd(botname[i], "MODE %s +o %s", chan, botname[i]);

		}

	    } else if (stricmp(cmd, "ALLBOTPART") == 0 ) {
		int i;
	        chan = strtok(NULL, " ");
		if (!is_services_root(source)) {
			notice(s_MassServ, source, "Permission denied.");
		       	return;
		}
	        if (!chan) {
		    notice(s_MassServ, source, "Syntax: ALLBOTPART <channel>");
	            return;
		}
		for (i=1; i<=servbot; i++)
		        send_cmd(botname[i], "PART %s", chan);

            } else if (stricmp(cmd, "ALLBOTQUIT") == 0) {
		int i;
		if (!is_services_coder(source)) {
			notice(s_MassServ, source, "Permission denied.");
		       	return;
		}
	        wallops(s_BotServ, "All Services bot quited by \2%s\2", source);
		for (i=1; i<=servbot; i++) {
	                send_cmd (botname[i], "QUIT :Quit by %s", source);
			botname[i] = NULL;
			botmaster[i] = NULL;
		}
		servbot = 0;
            } else if (stricmp(cmd, "ALLJOIN") == 0) {
	        chan = strtok(NULL, " ");
		if (!is_services_coder(source)) {
			notice(s_MassServ, source, "Permission denied.");
		       	return;
		}
	        if (!chan) {
		    notice(s_MassServ, source, "Syntax: ALLJOIN <channel>");
	            return;
		}
		alljoin(chan);
	    } else if (stricmp(cmd, "HELP") == 0) {
		notice(s_MassServ, source,"Mass Services still under construction.");
		notice(s_MassServ, source,"- SERVJOIN      Make all Services bots Join a channel");
		notice(s_MassServ, source,"- SERVPART      Make all Services bots Part a channel");
		notice(s_MassServ, source,"- ALLBOTJOIN    Make all bots Join a channel");
		notice(s_MassServ, source,"- ALLBOTPART    Make all bots Part a channel");		
		notice(s_MassServ, source,"- ALLBOTQUIT    Exit all bots");
		notice(s_MassServ, source,"- ALLJOIN       Make all users join a channel");
	    } else
		notice(s_MassServ, source,"Mass Services still under construction.");
}
/************************************************************************/
void sbotserv(const char *source, char *buf)
{
    char *cmd, *s, *chan, *bchan;
    char orig[BUFSIZE];
    strscpy (orig, buf, sizeof (orig));
    cmd = strtok(buf, " ");
//    log(buf);
    if (cmd) 
	    if (stricmp (cmd, "\1PING") == 0)
	    {
	        if (!(s = strtok (NULL, "")))
	            s = "\1";
	        notice(s_BotServ, source, "\1PING %s", s);

	    } else if (stricmp (cmd, "\1VERSION\1") == 0) {
	        if (!(s = strtok (NULL, "")))
        	    s = "\1";
	        notice(s_BotServ, source, "%sVERSION Auspice IRC Services %s", s, s);

	    } else if (stricmp(cmd, "LIST") == 0) {
		int i;
	        notice(s_BotServ, source, "\2%d BOT(s) IN LIST\2", servbot);
		for(i=1; i <= servbot; i++)
		        notice(s_BotServ, source, "%d)- %s (%s)",i, botname[i], botmaster[i]);
	        notice(s_BotServ, source, "\2ENF OF LIST\2");

	    } else if (!is_services_admin(source)) {

       		notice(s_BotServ, source, "Permision Denied.");
	       	return;

	    } else if (stricmp(cmd, "JOIN") == 0) {

		if (!is_services_admin(source)) {
	       		notice(s_BotServ, source, "Permision Denied.");
		       	return;
		}
		chan = strtok(NULL, " ");
		if (!chan) {
			notice(s_BotServ, source, "Syntax: JOIN \2Channel\2");
		} else {
		        send_cmd (s_BotServ, "JOIN %s", chan);
		}

	    } else if (stricmp(cmd, "PART") == 0) {
		if (!is_services_admin(source)) {
	       		notice(s_BotServ, source, "Permision Denied.");
		       	return;
		}
		chan = strtok(NULL, " ");
		if (!chan) {
			notice(s_BotServ, source, "Syntax: PART \2Channel\2");
		} else {
		        send_cmd (s_BotServ, "PART %s", chan);
		}
	    } else if (stricmp(cmd, "WHOIS") == 0) {
		if (!is_services_admin(source)) {
	       		notice(s_BotServ, source, "Permision Denied.");
		       	return;
		}
		chan = strtok(NULL, " ");
		if (!chan) {
			notice(s_BotServ, source, "Syntax: WHOIS \2Botnick\2");
		} else {
			int i;
			for (i=1; i<=servbot; i++)
			if (stricmp(chan, botname[i]) == 0) {
				notice(s_BotServ, source,"WHOIS: Bot %s connected by %s", botname[i], botmaster[i]);
				return;
			}
		}

	    } else if (stricmp(cmd, "SNOOP") == 0) {
		if (!is_services_root(source)) {
	       		notice(s_BotServ, source, "Permision Denied.");
		       	return;
		}
		chan = strtok(NULL, " ");
		bchan = strtok(NULL, " ");
		if (chan && (stricmp(chan,"NONE") == 0)) {
			aspychan = 0;
			notice(s_BotServ, source, "Disable Channel Spy feature");
		} else if (!chan || !bchan) {
			notice(s_BotServ, source, "Syntax: SNOOP \2source\2 \2dest\2");
		} else if (chan && !bchan) {
			notice(s_BotServ, source, "Syntax: SNOOP \2source\2 \2dest\2");
		} else {
			asource = sstrdup(chan);
			adest = sstrdup(bchan);
			aspychan = 1;
			notice(s_BotServ, source, "All message in channel %s forward to %s.", chan, bchan);		
		}

	    } else if (stricmp(cmd, "CONNECT") == 0) {
		chan = strtok(NULL, " ");
		if (!is_services_admin(source)) {
	       		notice(s_BotServ, source, "Permision Denied.");
		       	return;
		}
		if (!chan) {
			notice(s_BotServ, source, "Syntax: CONNECT \2nick\2");
		} else {
			if (finduser(chan) || (is_servbot(chan) == 1)) {
				notice(s_BotServ, source, "Current nick is using.");
			} else {
//			NEWNICK(chan, "ServerBot", services_host, server_name, "+rdqBS", 1);
			send_cmd (NULL, "NICK %s 1 %lu %s %s %s :%s",
	                chan, time(NULL), "ServerBot", services_host, server_name, "Services Bot");
		        send_cmd(chan, "MODE %s +qB", chan);
		        send_cmd(chan, "JOIN %s", snoopchan);
	                send_cmd(chan,"SWHOIS %s :is a Services Bot",chan);
		        wallops(s_BotServ, "Services bot \2%s\2 connected by \2%s\2", chan, source);
			++servbot;
			botname[servbot] = sstrdup(chan);
			botmaster[servbot] = sstrdup(source);
			}
		}

	    } else if (stricmp(cmd, "HELP") == 0) {
		notice(s_BotServ, source,"Bot Services still under construction.");
		notice(s_BotServ, source, "- CONNECT Connect Services Bot.");
		notice(s_BotServ, source, "- LIST    List all Services Bot.");
		notice(s_BotServ, source, "- WHOIS   Do whois a bot.");
		notice(s_BotServ, source, "- SNOOP   Spy channel.");
	    } else
		notice(s_BotServ, source,"Bot Services still under construction.");
}
/********************************************************************/
void bmsgchan(const char *source, char *chan, char *buf)
{
	int c;
	ChannelInfo *ci;
	ci = cs_findchan(chan);

	if ((aspychan == 1) && (stricmp(chan,asource) == 0))
	     send_cmd(s_OperServ, "PRIVMSG %s :<%s/%s> %s", adest, source, chan, buf);

	if (!ci || !source) {

	} else if (ci && !(ci->bot)) {

	} else {
	if (!((ci->botflag & CBI_PROTOP) && (is_voiced(source,chan) || is_chanop(source, chan) || is_chanhop(source, chan) || is_prots(source, chan)))) {
	        char *nick = sstrdup(source);

		/***************** CAP *******************/
		if ((ci->botflag & CBI_KICKCAP) && (c = strlen(buf) > 10)) {
			int i = 0;
		        char *s = buf;

		        do {
		        	if (isupper(*s)) i++;
			} while (*s++);
			c = strlen(buf);
			if (i >= 10 && i*100/c>=25)
			{
				if (is_chanhop(source, ci->name) || is_chanop(source, ci->name)) {
					notice(ci->bot, source,"Your caplocks excess the limit please drop it.");
					dropmode(nick, ci);
					return;
				} else {
					botkick(nick, ci, "Please turn caps lock OFF!");
					return;
				}
			}
		}
		/********** END CAP **************/

                /********** BOLDS ****************/
		if ((ci->botflag & CBI_KICKBOLDS) && strchr(buf, 2)) {
			if (is_chanhop(source, ci->name) || is_chanop(source, ci->name)) {
				notice(ci->bot, source,"Don't use bolds on this channel!.");
				dropmode(nick, ci);
				return;
			} else {
				botkick(nick, ci, "Don't use bolds on this channel!");
				return;
			}
		}
                /***************** END BOLD **************/
		/***************** COLORS ****************/
		if ((ci->botflag & CBI_KICKCOLORS) && strchr(buf, 3)) {
			if (is_chanhop(source, ci->name) || is_chanop(source, ci->name)) {
				notice(ci->bot, source,"Don't use colors on this channel!");
				dropmode(nick, ci);
				return;
			} else {
				botkick(nick, ci, "Don't use colors on this channel!");
				return;
			}
		}
                /******************** END COLOR ******************/           
		/******************** REVERSES *******************/
		if ((ci->botflag & CBI_KICKREVERSES) && strchr(buf, 22)) {
			if (is_chanhop(source, ci->name) || is_chanop(source, ci->name)) {
				notice(ci->bot, source,"Don't use reverses on this channel!");
				dropmode(nick, ci);
				return;
			} else {
				botkick(nick, ci, "Don't use reverses on this channel!");
				return;
			}
		}
		/******************** END REVERSES *******************/
		/******************** UNDERLINES *********************/
		if ((ci->botflag & CBI_KICKUNDERLINES) && strchr(buf, 31)) {
			if (is_chanhop(source, ci->name) || is_chanop(source, ci->name)) {
				notice(ci->bot, source,"Don't use underlines on this channel!");
				dropmode(nick, ci);
				return;
			} else {
				botkick(nick, ci, "Don't use underlines on this channel!");
				return;
			}
		}
		/******************* END UNDERLINES ******************/
		/******************** REPEAT *********************/
/*
		if (ci->botflag & CBI_KICKREPEAT) {
			UserData *ud;
			User *u = finduser(source);
			Channel *c = findchan(ci->name);
			if (!u || !c)
				return;

			ud = get_user_data(c, u);

                        if (!ud) return;

                        if (ud->lastline && stricmp(ud->lastline, buf)) {
                                free(ud->lastline);
                                ud->lastline = sstrdup(buf);
                                ud->times = 0;
                        } else {
                                if (!ud->lastline) ud->lastline = sstrdup(buf);
                                ud->times++;
                        }
                        if (ud->times >= 3)
                        {
				if (is_chanhop(source, ci->name) || is_chanop(source, ci->name)) {
					notice(ci->bot, source,"Stop repeat!");
					dropmode(nick, ci);
					return;
				} else {
					botkick(nick, ci, "Don't repeat in this channel!");
					return;
				}
			}
		} */
		/******************* END REPEAT ******************/
		/******************** FLOOD *********************/
/*
		if (ci->botflag & CBI_KICKFLOOD) {
			UserData *ud;
			time_t now = time(NULL);
			User *u = finduser(source);
			Channel *c = findchan(ci->name);

			if (!u || !c || !now)
				return;

			ud = get_user_data(c, u);

                        if (!ud) return;

                        if (now - ud->last_start > 2)
                        {
                                ud->last_start = time(NULL);
                                ud->lines = 0;
                        }

                        ud->lines++;

                        if (ud->lines >= 5)
                        {
				if (is_chanhop(source, ci->name) || is_chanop(source, ci->name)) {
					notice(ci->bot, source,"Stop Flooding!");
					dropmode(nick, ci);
					return;
				} else {
					botkick(nick, ci, "Don't flood in this channel!");
					return;
				}
			}
		} */
		/******************* END FLOOD ******************/

	}	
		/******************** BOT COMMAND ****************/
		if (ci->botflag & CBI_BOTCOM) {
			abotcom(source, ci, buf);
		}
	}
}
/************************************************************************/
void botkick(char *nick, ChannelInfo *ci, char *buf)
{
	if (is_on_chan(nick, ci->name)) {
	       User *u2 = finduser(nick);
	       char *av[3];
               av[0] = sstrdup(ci->name);
               av[1] = sstrdup("+b");
	       av[2] = bcreate_mask(u2, 1);
			
	       send_cmd(ci->bot, "MODE %s +b %s  %lu", ci->name, av[2], time(NULL));
	       do_cmode(ci->bot, 3, av);

                av[1] = sstrdup(nick);
		av[2] = sstrdup(buf);
		send_cmd(ci->bot, "KICK %s %s :%s", av[0], av[1], av[2]);
		do_kick(ci->bot, 3, av);
                free(av[2]);
                free(av[1]);
                free(av[0]);
	}		
}
/************************************************************************/
void dropmode(char *nick, ChannelInfo *ci)
{
	if (is_on_chan(nick, ci->name)) {
		if (is_chanop(nick, ci->name))
		change_cmode (ci->bot, ci->name, "-o", nick);
		if (is_prots(nick, ci->name))
		change_cmode (ci->bot, ci->name, "-a", nick);
		if (is_chanhop(nick, ci->name))
		change_cmode (ci->bot, ci->name, "-h", nick);
		if (is_voiced(nick, ci->name))
		change_cmode (ci->bot, ci->name, "-v", nick);
	}		
}
/************************************************************************/
static void abotcom(const char *source, ChannelInfo *ci, char *buf)
{
   char *chan = ci->name;
   User *u = finduser (source);
   char *cmd, *param;
   cmd = strtok(buf, " ");
		
   if(u && cmd) {
        /******************** OP Command ********************/

	if ((stricmp(cmd,"!op")==0) || (stricmp(cmd,"!up")==0)) {
	   param = strtok(NULL, " ");

	   if (!is_chanop(source, chan) && (get_access(u, ci) < 5)) {
		notice(ci->bot, source,"You don't have access.");
		return;
	   } 

	  if (!param) {
		if (!is_on_chan(source, chan))
			notice(ci->bot, source,"You are not in channel %s.", chan);
		else if (is_chanop(source, chan))
			notice(ci->bot, source,"You already have op in %s.", chan);
		else
		        change_cmode (ci->bot, chan, "+o", source);

	  } else {
		User *u2 = finduser(param);
		if (!u2 || !is_on_chan(param, chan))
			notice(ci->bot, source,"User %s is not in %s.",param,chan);
		else if ((ci->flags & CI_OPGUARD) && (get_access(u2, ci) < 5))
			notice(ci->bot, source,"Channel %s set opguard on please respect founder.",chan);
		else if (is_chanop(param, chan))
			notice(ci->bot, source,"User %s already have op in %s.",param,chan);
		else
		        change_cmode (ci->bot, chan, "+o", param);
	  }

	}

       /********************* END OP *************************/
       /********************** DEOP *************************/
	else if ((stricmp(cmd,"!deop")==0) || (stricmp(cmd,"!down")==0)) {
	   param = strtok(NULL, " ");
	   if (!is_chanop(source, chan) && (get_access(u, ci) < 5)) {
		notice(ci->bot, source,"You don't have access.");
		return;
	   }

	  if (!param) {
		if (!is_on_chan(source, chan))
			notice(ci->bot, source,"You are not in channel %s.", chan);
		else if (!is_chanop(source, chan))
			notice(ci->bot, source,"You already deop in %s.", chan);
		else
		        change_cmode (ci->bot, chan, "-o", source);

	  } else {
		if (!is_on_chan(param, chan))
			notice(ci->bot, source,"User %s is not in %s.",param,chan);
		else if (!is_chanop(param, chan))
			notice(ci->bot, source,"User %s already deop in %s.",param,chan);
		else
		        change_cmode (ci->bot, chan, "-o", param);
	  }
       }
         /*************************** End Deop *********************/
         /*************************** molest ***********************/
	else if (stricmp(cmd,"!molest")==0) {

	if (!is_on_chan(source, chan))
		notice(ci->bot, source,"You are not in channel %s.", chan);
	else
		dropmode(sstrdup(source), ci);

       }
         /*************************** End MoLest *******************/
         /*************************** Voice ************************/
	else if (stricmp(cmd,"!voice")==0) {
	   param = strtok(NULL, " ");

	   if (!is_chanop(source, chan) && (get_access(u, ci) < 4)) {
		notice(ci->bot, source,"You don't have access.");
		return;
	   }

	  if (!param) {
		if (!is_on_chan(source, chan))
			notice(ci->bot, source,"You are not in channel %s.", chan);
		else if (is_voiced(source, chan))
			notice(ci->bot, source,"You already have voice in %s.", chan);
		else
		        change_cmode (ci->bot, chan, "+v", source);

	  } else {
		if (!is_on_chan(param, chan))
			notice(ci->bot, source,"User %s is not in %s.",param,chan);
		else if (is_voiced(param, chan))
			notice(ci->bot, source,"User %s already have voice in %s.",param,chan);
		else
		        change_cmode (ci->bot, chan, "+v", param);
	  }
	}
         /*************************** End Voice ********************/
         /*************************** De Voice *********************/
	else if (stricmp(cmd,"!devoice")==0) {
	   param = strtok(NULL, " ");

	   if (!is_chanop(source, chan) && (get_access(u, ci) < 4)) {
		notice(ci->bot, source,"You don't have access.");
		return;
	   }

	  if (!param) {
		if (!is_on_chan(source, chan))
			notice(ci->bot, source,"You are not in channel %s.", chan);
		else if (!is_voiced(source, chan))
			notice(ci->bot, source,"You already devoice in %s.", chan);
		else
		        change_cmode (ci->bot, chan, "-v", source);

	  } else {
		if (!is_on_chan(param, chan))
			notice(ci->bot, source,"User %s is not in %s.",param,chan);
		else if (!is_voiced(param, chan))
			notice(ci->bot, source,"User %s already devoice in %s.",param,chan);
		else
		        change_cmode (ci->bot, chan, "-v", param);
	  }
	}
         /*************************** End DeVoice ******************/
         /*************************** Halfop ***********************/
	else if (stricmp(cmd,"!halfop")==0) {
	   param = strtok(NULL, " ");

	   if (!is_chanop(source, chan) && (get_access(u, ci) < 5)) {
		notice(ci->bot, source,"You don't have access.");
		return;
	   }

	  if (!param) {
		if (!is_on_chan(source, chan))
			notice(ci->bot, source,"You are not in channel %s.", chan);
		else if (is_chanhop(source, chan))
			notice(ci->bot, source,"You already have halfop in %s.", chan);
		else
		        change_cmode (ci->bot, chan, "+h", source);

	  } else {
		User *u2 = finduser(param);
		if (!is_on_chan(param, chan))
			notice(ci->bot, source,"User %s is not in %s.",param,chan);
		else if ((ci->flags & CI_OPGUARD) && (get_access(u2, ci) < 4))
			notice(ci->bot, source,"Channel %s set opguard on please respect founder.",chan);
		else if (is_chanhop(param, chan))
			notice(ci->bot, source,"User %s already have halfop in %s.",param,chan);
		else
		        change_cmode (ci->bot, chan, "+h", param);
	  }
	}
         /*************************** End Halfop *******************/
	else if (stricmp(cmd,"!dehalfop")==0) {
	   param = strtok(NULL, " ");

	   if (!is_chanop(source, chan) && (get_access(u, ci) < 5)) {
		notice(ci->bot, source,"You don't have access.");
		return;
	   }

	  if (!param) {
		if (!is_on_chan(source, chan))
			notice(ci->bot, source,"You are not in channel %s.", chan);
		else if (!is_chanhop(source, chan))
			notice(ci->bot, source,"You already dehalfop in %s.", chan);
		else
		        change_cmode (ci->bot, chan, "-h", source);

	  } else {
		if (!is_on_chan(param, chan))
			notice(ci->bot, source,"User %s is not in %s.",param,chan);
		else if (!is_chanhop(param, chan))
			notice(ci->bot, source,"User %s already dehalfop in %s.",param,chan);
		else
		        change_cmode (ci->bot, chan, "-h", param);
	  }

	}
         /*************************** DeHalfop *********************/
         /*************************** Kick *************************/
	else if (stricmp(cmd,"!kick")==0) {
	   char *reason;
	   param = strtok(NULL, " ");
	   reason = strtok(NULL, " ");

	   if (!is_chanop(source, chan) && (get_access(u, ci) < 4)) {
		notice(ci->bot, source,"You don't have access.");
		return;
	   }

	  if (!param) {
		notice(ci->bot, source,"Syntax: !KICK <nick> [<reason>]");
	  } else {
		if (!is_on_chan(param, chan))
			notice(ci->bot, source,"User %s is not in %s.",param,chan);
		else if (is_chanhop(param, chan) || is_chanop(param, chan) || is_prots(param, chan))
			notice(ci->bot, source,"I don't kick op or protected user.");
		else {
		       char *av[3];
	               av[0] = sstrdup(chan);
	               av[1] = sstrdup(param);
	               if (!reason)
		          av[2] = sstrdup(buf);
	               else
	                  av[2] = sstrdup(reason);
	               send_cmd(ci->bot, "KICK %s %s :%s", av[0], av[1], av[2]);
	               do_kick(ci->bot, 3, av);
	               free(av[2]);
	               free(av[1]);
	               free(av[0]);
                }		
	  }
	}
         /************************** End Kick **********************/
         /************************** Ban ***************************/
	else if (stricmp(cmd,"!ban")==0) {
	   char *btype;
	   param = strtok(NULL, " ");
	   btype = strtok(NULL, " ");

	   if (!is_chanop(source, chan) && (get_access(u, ci) < 4)) {
		notice(ci->bot, source,"You don't have access.");
		return;
	   }

	  if (!param) {
		notice(ci->bot, source,"Syntax: !BAN <nick> ![<type>]");
	  } else {
		if (!is_on_chan(param, chan))
			notice(ci->bot, source,"User %s is not in %s.",param,chan);
		else if (is_chanhop(param, chan) || is_chanop(param, chan) || is_prots(param, chan))
			notice(ci->bot, source,"I don't ban op or protected user.");
		else {
		       User *u2 = finduser(param);
		       char *av[3];
	               av[0] = sstrdup(chan);
	               av[1] = sstrdup("+b");

		       if (btype && *btype == '!')
	               if (!btype || (stricmp(btype,"!1")==0))
		          av[2] = bcreate_mask(u2, 1);
	               else
	                  av[2] = bcreate_mask(u2, 2);
		       else
		          av[2] = bcreate_mask(u2, 1);

		       send_cmd(ci->bot, "MODE %s +b %s  %lu", chan, av[2], time(NULL));
		       do_cmode(s_ChanServ, 3, av);
	               free(av[2]);
	               free(av[1]);
	               free(av[0]);
                }		
	  }
	}
         /************************** End Ban ***********************/
         /************************** BKICK *************************/
	else if (stricmp(cmd,"!bkick")==0) {
	   char *reason, *btype, newreason[512];
	   param = strtok(NULL, " ");
	   btype = strtok(NULL, " ");
	   reason = strtok(NULL, "");

	   if (!is_chanop(source, chan) && (get_access(u, ci) < 4)) {
		notice(ci->bot, source,"You don't have access.");
		return;
	   }

	  if (!param) {
		notice(ci->bot, source,"Syntax: !BKICK <nick> ![<type>] [<reason>]");
	  } else {
		if (!is_on_chan(param, chan))
			notice(ci->bot, source,"User %s is not in %s.",param,chan);
		else if (is_chanhop(param, chan) || is_chanop(param, chan) || is_prots(param, chan))
			notice(ci->bot, source,"I don't ban kick op or protected user.");
		else {
		       User *u2 = finduser(param);
		       char *av[3];
	               av[0] = sstrdup(chan);
	               av[1] = sstrdup("+b");

		       if (btype && *btype == '!')
	               if (!btype || (stricmp(btype,"!1")==0))
		          av[2] = bcreate_mask(u2, 1);
	               else
	                  av[2] = bcreate_mask(u2, 2);
		       else
		          av[2] = bcreate_mask(u2, 1);
			
		       send_cmd(ci->bot, "MODE %s +b %s  %lu", chan, av[2], time(NULL));
		       do_cmode(ci->bot, 3, av);

	               av[1] = sstrdup(param);

	               if (!reason)
		          av[2] = sstrdup("Ban kick requested");
	               else {
		          if (!(*btype == '!')) {
				  *newreason = 0;
				  strcat(newreason, btype);
				  strcat(newreason, " ");
				  strcat(newreason, reason);
		                  av[2] = sstrdup(newreason);
			   } else
		                  av[2] = sstrdup(reason);

		       }
	               send_cmd(ci->bot, "KICK %s %s :%s", av[0], av[1], av[2]);
	               do_kick(ci->bot, 3, av);

	               free(av[2]);
	               free(av[1]);
	               free(av[0]);
                }		
	  }
	}
         /************************** Ban Kick **********************/
         /************************** Un Ban ************************/
	else if (stricmp(cmd,"!unban")==0) {
	   char *btype;
	   param = strtok(NULL, " ");
	   btype = strtok(NULL, " ");

	   if (!is_chanop(source, chan) && (get_access(u, ci) < 4)) {
		notice(ci->bot, source,"You don't have access.");
		return;
	   }

	  if (!param) {
		notice(ci->bot, source,"Syntax: !UNBAN <nick>");
	  } else {
	       User *u2 = finduser(param);
	       Channel *c= findchan(chan);
	       char *av[3];
	       int i;
	       if (c && u2) {
	               av[0] = sstrdup(chan);
	               av[1] = sstrdup("-b");
		       for (i = c->bancount-1; i > -1; --i) {
		            if (match_usermask(c->bans[i], u2)) {
		                send_cmd(ci->bot, "MODE %s -b %s", chan, c->bans[i]);
		                av[2] = sstrdup(c->bans[i]);
		                do_cmode(s_ChanServ, 3, av);
		                free(av[2]);
		            }
		        }

	               free(av[1]);
        	       free(av[0]);
		}
	  }
	}
         /************************** End Unban *********************/
         /************************** HELP **************************/
	else if (stricmp(cmd,"!help")==0) {

	   if (!is_chanop(source, chan) && (get_access(u, ci) < 4)) {
		notice(ci->bot, source,"You don't have access.");
		return;
	   }

		if (!is_on_chan(source, chan))
			notice(ci->bot, source,"You are not in channel %s.", chan);
		else {
		notice(ci->bot, source,"!op       \2[<nick>]\2");
		notice(ci->bot, source,"!deop     \2[<nick>]\2");
		notice(ci->bot, source,"!voice    \2[<nick>]\2");
		notice(ci->bot, source,"!devoice  \2[<nick>]\2");
		notice(ci->bot, source,"!halfop   \2[<nick>]\2");
		notice(ci->bot, source,"!dehalfop \2[<nick>]\2");
		notice(ci->bot, source,"!kick     \2<nick>\2 \2[<reason>]\2");
		notice(ci->bot, source,"!ban      \2<nick>\2 \2![<type = 1|2>]\2");
		notice(ci->bot, source,"!bkick    \2<nick>\2 \2![<type = 1|2>]\2 \2[<reason>]\2");
		notice(ci->bot, source,"End of bot command.");
	}
       }
         /************************** End Help **********************/
   } else {
   }
}

/******************************************************************/
/*
static UserData *get_user_data(Channel *c, User *u)
{
        struct c_userlist *user;

        if (!c || !u) return NULL;

        for (user = c->users; user; user = user->next) {
                if (user->user == u) {
                        if (user->ud)
                        {
                                time_t now = time(NULL);

                                if (now - user->ud->last_use > 10)
                                {
		                        if (user->ud->lastline) free(user->ud->lastline);

                                        memset(user->ud, 0, sizeof(UserData));
                                        user->ud->last_use = now;
                                }

                                return user->ud;
                        } else {
                                user->ud = scalloc(sizeof(UserData), 1);
                                user->ud->last_use = time(NULL);
                                return user->ud;
                        }

                }
        }

        return NULL;
}

*/
/**********************************************************************/
int sockcheck2(char *host)
{
  struct hostent *hp;
  struct sockaddr_in sa;
  int sock = -1;
  unsigned char packet[12];

  if (!(hp = gethostbyname(host))) return;


  if ((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1) return;

  bzero(&sa, sizeof(sa));
  sa.sin_family = AF_INET;
  sa.sin_port = htons (SOCKS5PORT);
  bcopy(hp->h_addr, (char *) &sa.sin_addr, hp->h_length);
  if (connect(sock, (struct sockaddr *) &sa, sizeof (sa)) == 0) {
    packet[0] = 5;
    packet[1] = 1;
    packet[2] = 0;

    if (debug)
        log("Checking for Unsecured socks5 for %s",host);
    if (write(sock, packet, 3) == 3) {
      bzero(packet, 2);
      if (read(sock, packet, 2) == 2 && packet[0]==5 && packet[1]==0) {
        close(sock);
	send_cmd(s_Agent, "GLINE *@%s 1800 :Unsecured socks5", host);
        return 1;
      }
    }
    close(sock);

    if ((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1) return;

    bzero(&sa, sizeof(sa));
    sa.sin_family = AF_INET;
    sa.sin_port = htons (SOCKS4PORT);
    bcopy(hp->h_addr, (char *) &sa.sin_addr, hp->h_length);
    if (debug)
        log("Checking for Unsecured socks4 for %s",host);
    if (connect(sock, (struct sockaddr *) &sa, sizeof (sa)) == 0) {
      unsigned long ip = htonl(sa.sin_addr.s_addr);
      packet[0] = 4;
      packet[1] = 1;
      packet[2] = (((unsigned short) SOCKS4PORT) >> 8);
      packet[3] = (((unsigned short) SOCKS4PORT) & 255);
      packet[4] = (ip >> 24);
      packet[5] = ((ip >> 16) & 255);
      packet[6] = ((ip >> 8) & 255);
      packet[7] = (ip & 255);
      packet[8] = 0;
      if (write(sock, packet, 9) == 9) {
        bzero(packet, 9);
        if (read(sock, packet, 9)>=4 && packet[1] == 90) {
          close(sock);
	  send_cmd(s_Agent, "GLINE *@%s 1800 :Unsecured socks4", host);
          return 1;
        }
      }
    }
  }
  close(sock);


  if ((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1) return;
  bzero(&sa, sizeof(sa));
  sa.sin_family = AF_INET;
  sa.sin_port = htons(6667);
  bcopy(hp->h_addr, (char *) &sa.sin_addr, hp->h_length);
  if (connect(sock, (struct sockaddr *) &sa, sizeof (sa)) == 0) {
    memset(packet, 0, sizeof(packet));
    if (read(sock, packet, 10)) {
        wallops(s_Agent, "Possible an IRC Server detected from: \2%s\2", host);
        close(sock);
    }
  }
  close(sock);

  if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) return;

  bzero(&sa, sizeof(sa));
  sa.sin_family = AF_INET;
  sa.sin_port = htons(WINGATE);
  bcopy(hp->h_addr, (char *) &sa.sin_addr, hp->h_length);

  if (debug)
        log("Checking for Unsecured wingate for %s",host);

  if (connect(sock, (struct sockaddr *) &sa, sizeof (sa)) == 0) {
    memset(packet, 0, sizeof(packet));
    if (read(sock, packet, 8)==8
        && (!strncasecmp(packet, "WinGate>", 8)
            || !strncasecmp(packet, "Too many", 8))) {
        close(sock);
	send_cmd(s_Agent, "GLINE *@%s 1800 :Unsecured wingate", host);
      return 1;
    }
  }
  close(sock);
  return 0;
}
/************************************************************************/
