/* Routines to maintain a list of online users.
 * Auspice Services is copyright (c) 2000-2001 In Mean.
 *     E-mail: <inmean@khmerpop.com>
 *
 * SirvNET Services is copyright (c) 1998-2000 Trevor Klingbeil.
 *     E-mail: <priority1@dal.net>
 * Originally based on EsperNet Services(c) by Andy Church.
 * This program is free but copyrighted software; see the file LICENSE for
 * details.
 */

#include "../inc/services.h"

User *userlist = NULL;

int usercnt = 0, opcnt = 0, maxusercnt = 0, connectscnt = 0;

/*************************************************************************/

/* Allocate a new User structure, fill in basic values, link it to the
 * overall list, and return it.  Always successful.
 */

static User *new_user(const char *nick)
{
    User *user;
    NickInfo *ni;

    user = scalloc(sizeof(User), 1);
    if (!nick)
	nick = "";
    strscpy(user->nick, nick, NICKMAX);
    user->floodlev = 0;
    user->next = userlist;
    if (userlist)
	userlist->prev = user;
    userlist = user;
    usercnt++;
    connectscnt++;
    if (usercnt > maxusercnt)
	maxusercnt = usercnt;

    m_sendmotd(nick);
    send_cmd(NULL, "USERHOST %s", nick);
    ni = findnick(nick);

    if (CUSTOM_NOTICE)
        notice(s_GlobalNoticer,nick,CUSTOM_WELCOME);
    if (!ni && GENERIC_NOTICE)
        notice(s_GlobalNoticer,nick, WELCOME_NOTICE,NETWORK_NAME,nick,NETWORK_NAME,
               s_NickServ, s_ChanServ);

    return user;
}

/*************************************************************************/

/* Change the nickname of a user, and move pointers as necessary. */

static void change_user_nick(User *user, const char *nick)
{
    if (spyhost)
    	if (stricmp(spyhost, user->host) == 0)
	    send_cmd(s_ChanServ, "PRIVMSG #Debug :%s -> %s (%s)", user->nick, nick, user->host);
    strscpy(user->nick, nick, NICKMAX);
}

/*************************************************************************/

/* Remove and free a User structure. */

void delete_user(User *user)
{
    struct u_chanlist *c, *c2;
    struct u_chaninfolist *ci, *ci2;
    int i;

    usercnt--;
    if (user->mode & UMODE_O)
	opcnt--;
    check_del_memos(user);
    cancel_user(user);
//    check_temp_os(user);
    free(user->username);
    free(user->host);
    free(user->realhost);
    free(user->realname);
    free(user->server);

    c = user->chans;
    while (c) {
	c2 = c->next;
	chan_deluser(user, c->chan);
	free(c);
	c = c2;
    }
    ci = user->founder_chans;

    while (ci) {
	ci2 = ci->next;
	free(ci);
	ci = ci2;
    }

    for (i=0; user->id_nicks[i] && i < MAX_IDS; i++)
        free(user->id_nicks[i]);

    if (user->prev)
	user->prev->next = user->next;
    else
	userlist = user->next;
    if (user->next)
	user->next->prev = user->prev;
    free(user);
}

/*************************************************************************/

/* Return statistics.  Pointers are assumed to be valid. */

void get_user_stats(long *nusers, long *memuse)
{
    long count = 0, mem = 0;
    User *user;
    struct u_chanlist *uc;
    struct u_chaninfolist *uci;
    int i;

    for (user = userlist; user; user = user->next) {
	count++;
	mem += sizeof(*user);
	if (user->username)
	    mem += strlen(user->username)+1;
	if (user->host)
	    mem += strlen(user->host)+1;
	if (user->realname)
	    mem += strlen(user->realname)+1;
	if (user->server)
	    mem += strlen(user->server)+1;
	for (uc = user->chans; uc; uc = uc->next)
	    mem += sizeof(*uc);
	for (uci = user->founder_chans; uci; uci = uci->next)
	    mem += sizeof(*uci);
        for (i=0; user->id_nicks[i] && i < MAX_IDS; i++)
            mem += strlen(user->id_nicks[i]);
    }
    *nusers = count;
    *memuse = mem;
}

/*************************************************************************/

/* Checks realname against list provided, if matches wallops. */

static char *check_realname(User *user)
{

   char *realname;

   if (match_wild_nocase("admin", user->realname))
      sprintf(realname,"%s", user->realname);
   
   else if (match_wild_nocase("ircop", user->realname))
      sprintf(realname,"%s", user->realname);

   else if (match_wild_nocase("service", user->realname))
      sprintf(realname,"%s", user->realname);

   else
      sprintf(realname,"");

  return realname;
}

/*************************************************************************/

/* Find a user by nick.  Return NULL if user could not be found. */

User *finduser(const char *nick)
{
    User *user = userlist;
    while (user && stricmp(user->nick, nick) != 0)
	user = user->next;
    return user;
}

int is_on_id_list(const char *source, const char *nick)
{
    User *u = finduser(source);
    int i;
    if (!u)
	return 0;

    for (i=0; i < MAX_IDS && u->id_nicks[i]; i++) {
	if (stricmp(u->id_nicks[i], nick) == 0) {
	    return 1;
	}
    }

    return 0;
}

/*************************************************************************/

/* Return the first user in the user list. */

User *first_user(void)
{
    return userlist;
}

/*************************************************************************/
/*************************************************************************/

/* Handle a server NICK command.
 *	av[0] = nick
 *	If a new user:
 *		av[1] = hop count
 *		av[2] = signon time
 *		av[3] = username
 *		av[4] = hostname
 *		av[5] = user's server
 *		av[6] = user's real name
 *	Else:
 *		av[1] = time of change
 */

void do_nick(const char *source, int ac, char **av)
{
    char *uv[3];
    User *user;
    if (!*source) {
	/* This is a new user; create a User structure for it. */

	if (debug)
	    log("debug: new user: %s", av[0]);
	/* We used to ignore the ~ which a lot of ircd's use to indicate no
	 * identd response.  That caused channel bans to break, so now we
	 * just take what the server gives us.  People are _still_
	 * encouraged to read the RFCs and stop doing anything to usernames
	 * depending on the result of an identd lookup.
	 */

	/* First check for AKILLs. */
	if (check_akill(av[0], av[3], av[4]))
	    return;
	if (asockcheck) {
        notice(s_Agent, av[0], "Checking for any Unsecure Sockets or Wingate...");
        if(sockcheck2(av[0]))
		return;
	else
	        notice(s_Agent, av[0], "No any Unsecure Sockets or Wingate found, GOOD!!!");
	}

	/* Allocate User structure and fill it in. */
	user = new_user(av[0]);
	user->signon = atol(av[2]);
	user->username = sstrdup(av[3]);
	user->host = sstrdup(av[4]);
	user->realhost = sstrdup(av[4]);
	user->server = sstrdup(av[5]);
	user->realname = sstrdup(av[6]);

	/* Check to see if it looks like clones. */
        if (clone_detect == 1)
           check_clones(user);

        /* Check to see if realname setting matches against warning list */

     if (realname == 1) {
        if ((match_wild_nocase("*admin*", user->realname))
        || (match_wild_nocase("*irc*op*", user->realname))
        || (match_wild_nocase("*service*", user->realname))
        || (match_wild_nocase("*oper*", user->realname)
        && (match_wild_nocase(NETWORK_NAME, user->realname)))
        && (stricmp(user->nick, "OperServ") != 0)
        && (stricmp(user->nick, "NickServ") != 0)
        && (stricmp(user->nick, "ChanServ") != 0)
        && (stricmp(user->nick, "HelpServ") != 0)
        && (stricmp(user->nick, "BotServ") != 0)
        && (stricmp(user->nick, "MemoServ") != 0)
        && (stricmp(user->nick, "RootServ") != 0)
        && (stricmp(user->nick, "OperServ2") != 0))

            wallops(s_OperServ,
                "\2Warning\2 - Real Name of \"\2%s\2\" found from %s!%s@%s",
                user->realname, user->nick, user->username, user->realhost);
    }

    do_logonmsg(user->nick);

    if (AUTOJOINC > 0)
	    send_cmd(s_Agent, "SVSJOIN %s %s", user->nick, ACHANS);

    do_autojoin(user->nick, user->realhost, user->username);

    } else {
	/* An old user changing nicks. */

	user = finduser(source);
	if (!user) {
	    log("user: NICK from nonexistent nick %s: %s", source,
							merge_args(ac, av));
		    return;
	}
	if (debug)
	    log("debug: %s changed nick to %s", source, av[0]);
	cancel_user(user);

	change_user_nick(user, av[0]);

	user->signon = atol(av[1]);

	if (strlen(av[0]) >= NICKMAX) {
	    send_cmd(SERVER_NAME, "KILL %s :Invalide nick for services.", av[0]);
	    do_kill(SERVER_NAME, 3, av);
	    return;
	}

    }
    user->my_signon = time(NULL);

    if (validate_user(user))
	check_memos(user->nick);
 
/*
    if ((stricmp(source, "guest") == 0) || (stricmp(source, "apodo") == 0) || (stricmp(source, "nickname") == 0)) {
            char newnick[NICKMAX];
            long int guestbuf = 0;
            double x = RAND_MAX + 1.0;
            int y;
            y = 1 + rand() * (99999 / x);
            guestbuf = y;
            snprintf (newnick, sizeof (newnick), "Guest%d", guestbuf);
            send_cmd(NULL, "SVSNICK %s %s :%lu", source, newnick, time(NULL));
    } else if ((stricmp(av[0], "guest") == 0) || (stricmp(av[0], "apodo") == 0) || (stricmp(av[0], "nickname") == 0)) {
            char newnick[NICKMAX];
            long int guestbuf = 0;
            double x = RAND_MAX + 1.0;
            int y;
            y = 1 + rand() * (99999 / x);
            guestbuf = y;
            snprintf (newnick, sizeof (newnick), "Guest%d", guestbuf);
            send_cmd(NULL, "SVSNICK %s %s :%lu", av[0], newnick, time(NULL));
    }
*/
}

/*************************************************************************/
/* Handle a JOIN command.
 *	av[0] = channels to join
 */

void do_join(const char *source, int ac, char **av)
{
    User *user;
    char *s, *t;
    struct u_chanlist *c;
    user = finduser(source);

    if (!user || !source) {
	log("user: JOIN from nonexistent user %s: %s", source,
							merge_args(ac, av));
	return;
    }
    t = av[0];
    while (*(s=t)) {
	t = s + strcspn(s, ",");
	if (*t)
	    *t++ = 0;
	if (debug)
	    log("debug: %s joins %s", source, s);

        /* Make sure check_kick comes before chan_adduser, so banned users
	 * don't get to see things like channel keys. */

	if (check_kick(user, s))
	    continue;

	check_welcome(user, s);
	chan_adduser(user, s);

	c = smalloc(sizeof(*c));
	c->next = user->chans;
	c->prev = NULL;
	if (user->chans)
	    user->chans->prev = c;
	user->chans = c;
	c->chan = findchan(s);
    }
}

/*************************************************************************/

/* Handle a PART command.
 *	av[0] = channels to leave
 *	av[1] = reason (optional)
 */

void do_part(const char *source, int ac, char **av)
{
    User *user;
    char *s, *t;
    struct u_chanlist *c;

    user = finduser(source);
    if (!user || !source) {
	log("user: PART from nonexistent user %s: %s", source,
							merge_args(ac, av));
	return;
    }
    t = av[0];
    while (*(s=t)) {
	t = s + strcspn(s, ",");
	if (*t)
	    *t++ = 0;
	if (debug)
	    log("debug: %s leaves %s", source, s);
	for (c = user->chans; c && stricmp(s, c->chan->name) != 0; c = c->next)
	    ;
	if (c) {
	    chan_deluser(user, c->chan);
	    if (c->next)
		c->next->prev = c->prev;
	    if (c->prev)
		c->prev->next = c->next;
	    else
		user->chans = c->next;
	    free(c);
	}
    }
}

/*************************************************************************/

/* Handle a KICK command.
 *	av[0] = channel
 *	av[1] = nick(s) being kicked
 *	av[2] = reason
 */

void do_kick(const char *source, int ac, char **av)
{
    User *user;
    char *s, *t;
    struct u_chanlist *c;

    t = av[1];
    while (*(s=t)) {
	t = s + strcspn(s, ",");
	if (*t)
	    *t++ = 0;
	user = finduser(s);
	if (!user) {
	    log("user: KICK for nonexistent user %s on %s: %s", s, av[0],
						merge_args(ac-2, av+2));
	    continue;
	}
	if (debug)
	    log("debug: kicking %s from %s", s, av[0]);

    if (spyhost)
	if (stricmp(spyhost, user->realhost))
	send_cmd(s_ChanServ, "PRIVMSG #Debug :kicking %s from %s", s, av[0]);

	for (c = user->chans; c && stricmp(av[0], c->chan->name) != 0;
								c = c->next)
	    ;
	if (c) {
	    chan_deluser(user, c->chan);
	    if (c->next)
		c->next->prev = c->prev;
	    if (c->prev)
		c->prev->next = c->next;
	    else
		user->chans = c->next;
	    free(c);
	}
    }
}

/*************************************************************************/

/* Handle a MODE command for a user.
 *	av[0] = nick to change mode for
 *	av[1] = modes
 */

void do_umode(const char *source, int ac, char **av)
{
    User *user;
    char *s;
    int add = 1;		/* 1 if adding modes, 0 if deleting */

    if (stricmp(source, av[0]) != 0) {
	log("user: MODE %s %s from different nick %s!", av[0], av[1], source);
	wallops(NULL, "%s attempted to change mode %s for %s",
		source, av[1], av[0]);
	return;
    }
    user = finduser(source);
    if (!user) {
	log("user: MODE %s for nonexistent nick %s: %s", av[1], source,
							merge_args(ac, av));
	return;
    }
    if (debug)
	log("debug: Changing mode for %s to %s", source, av[1]);
    if (spyhost)
	if (stricmp(spyhost, user->realhost))
    send_cmd(s_ChanServ, "PRIVMSG #Debug :%s MODE %s (%s)", source, av[1], user->host);

    s = av[1];
    while (*s) {
	switch (*s++) {
	    case '+': add = 1; break;
	    case '-': add = 0; break;
            case 'r':
                if (add && !is_on_id_list(source, user->nick)) {
                    send_cmd(s_NickServ, "SVSMODE %s -r", av[0]);
                }
                break;
	    case 'i': add ? (user->mode |= UMODE_I) : (user->mode &= ~UMODE_I); break;
	    case 'w': add ? (user->mode |= UMODE_W) : (user->mode &= ~UMODE_W); break;
	    case 'g': add ? (user->mode |= UMODE_G) : (user->mode &= ~UMODE_G); break;
	    case 's': add ? (user->mode |= UMODE_S) : (user->mode &= ~UMODE_S); break;
	    case 'A': add ? (user->mode |= UMODE_A) : (user->mode &= ~UMODE_A); break;
	    case 'N': add ? (user->mode |= UMODE_N) : (user->mode &= ~UMODE_N); break;
	    case 'T': add ? (user->mode |= UMODE_T) : (user->mode &= ~UMODE_T); break;
	    case 'B': add ? (user->mode |= UMODE_B) : (user->mode &= ~UMODE_B); break;
	    case 'o':
		if (add) {
                    if (!(user->flags & U_NOOPER)) {
			user->mode |= UMODE_O;

			if (OPERJOIN>0)
			    send_cmd(s_Agent, "SVSJOIN %s %s", user->nick, AOCHANS);
	      		do_operlogonmsg(user->nick);
                    	opcnt++;

		    } else {
			send_cmd(s_OperServ, "PRIVMSG #Snoop :Failed oper for %s", user->nick);
			send_cmd(NULL, "SVSMODE %s -oaA", user->nick);
			notice(s_OperServ, user->nick, "Sorry, You have the Abusive user mode set, You cannot oper.");
		    }

		} else {
		    user->mode &= ~UMODE_O;
		    --opcnt;
		}
		break;
	}
    }
}

/*************************************************************************/

/* Handle a QUIT command.
 *	av[0] = reason
 */

void do_quit(const char *source, int ac, char **av)
{
    User *user;
    NickInfo *ni;

    if (check_servii_kill(source, ac, av))
        return;

    user = finduser(source);
    if (!user || !source)
        return;
   
    if (debug)
	log("debug: %s quits", source);

    slog("Quit 1: %s",source);

    ni = findnick(source);

    if (ni && (!(ni->flags & NI_VERBOTEN)) && (ni->flags & (NI_IDENTIFIED | NI_RECOGNIZED)))
	ni->last_seen = time(NULL);

    slog("Quit 2: %s",source);

    delete_user(user);
    slog("Quit 3: %s",source);
}

/*************************************************************************/

/* Handle a KILL command.
 *	av[0] = nick being killed
 *	av[1] = reason
 */

void do_kill(const char *source, int ac, char **av)
{
    User *user;
    struct u_chanlist *c, *c2;
    struct u_chaninfolist *ci, *ci2;
    NickInfo *ni;

    if (check_servii_kill(source, ac, av))
        return;

    user = finduser(av[0]);
    if (!user)
        return;

    if (debug)
	log("debug: %s killed", av[0]);
    if ((ni = findnick(source)) && (!(ni->flags & NI_VERBOTEN)) && (ni->flags & (NI_IDENTIFIED | NI_RECOGNIZED)))
	ni->last_seen = time(NULL);
    delete_user(user);

}

/*************************************************************************/
/*************************************************************************/

/* Is the given nick an oper? */

int is_oper(const char *nick)
{
    User *user = finduser(nick);
    return user && (user->mode & UMODE_O);
}
/***********************************************************************/
int is_netadmin(const char *nick)
{
    User *user = finduser(nick);
    return user && (user->mode & UMODE_N);
}
/*************************************************************************/
int is_techadmin(const char *nick)
{
    User *user = finduser(nick);
    return user && (user->mode & UMODE_T);
}
/*************************************************************************/
int is_admin(const char *nick)
{
    User *user = finduser(nick);
    return user && (user->mode & UMODE_A);
}
/*************************************************************************/
int is_servadmin(const char *nick)
{
    User *user = finduser(nick);
    return user && (user->mode & UMODE_A);
}
/*************************************************************************/

/* Is the given nick on the given channel? */

int is_on_chan(const char *nick, const char *chan)
{
    User *u = finduser(nick);
    struct u_chanlist *c;

    if (!u)
	return 0;
    for (c = u->chans; c; c = c->next) {
	if (stricmp(c->chan->name, chan) == 0)
	    return 1;
    }
    return 0;
}

/*************************************************************************/

/* Is the given nick a channel operator on the given channel? */

int is_chanop(const char *nick, const char *chan)
{
    Channel *c = findchan(chan);
    struct c_userlist *u;

    if (!c)
	return 0;

    for (u = c->chanops; u; u = u->next) {

	if (stricmp(u->user->nick, nick) == 0)
	    return 1;
    }
    return 0;
}
/*************************************************************************/
int is_chanhop(const char *nick, const char *chan)
{
    Channel *c = findchan(chan);
    struct c_userlist *u;

    if (!c)
	return 0;

    for (u = c->halfop; u; u = u->next) {

	if (stricmp(u->user->nick, nick) == 0)
	    return 1;
    }
    return 0;
}
/*************************************************************************/

/* Is the given nick voiced (channel mode +v) on the given channel? */

int is_voiced(const char *nick, const char *chan)
{
    Channel *c = findchan(chan);
    struct c_userlist *u;

    if (!c)
	return 0;
    for (u = c->voices; u; u = u->next) {
	if (stricmp(u->user->nick, nick) == 0)
	    return 1;
    }
    return 0;
}
/*************************************************************************/
int is_prots(const char *nick, const char *chan)
{
    Channel *c = findchan(chan);
    struct c_userlist *u;

    if (!c)
	return 0;
    for (u = c->prots; u; u = u->next) {
	if (stricmp(u->user->nick, nick) == 0)
	    return 1;
    }
    return 0;
}
/***********************************************************************/
int is_cfounder(const char *nick, const char *chan)
{
    Channel *c = findchan(chan);
    struct c_userlist *u;

    if (!c)
	return 0;
    for (u = c->cfounder; u; u = u->next) {
	if (stricmp(u->user->nick, nick) == 0)
	    return 1;
    }
    return 0;
}

/*************************************************************************/
/*************************************************************************/

/* Does the user's usermask match the given mask (either nick!user@host or
 * just user@host)?
 */

int match_usermask(const char *mask, User *user)
{
    char *mask2 = sstrdup(mask);
    char *nick, *username, *host, *nick2, *host2;
    int result;

    if (strchr(mask2, '!')) {
	nick = strlower(strtok(mask2, "!"));
	username = strtok(NULL, "@");
    } else {
	nick = NULL;
	username = strtok(mask2, "@");
    }
    host = strtok(NULL, "");
    if (!username || !host) {
	free(mask2);
	return 0;
    }
    strlower(host);
    if (nick)
	nick2 = strlower(sstrdup(user->nick));
    host2 = strlower(sstrdup(user->host));
    result = (nick ? match_wild(nick, nick2) : 1) &&
		match_wild(username, user->username) &&
		match_wild(host, host2);
    free(mask2);
    if (nick)
	free(nick2);
    free(host2);
    return result;
}

/*************************************************************************/

/* Split a usermask up into its constitutent parts.  Returned strings are
 * malloc()'d, and should be free()'d when done with.  Returns "*" for
 * missing parts.
 */

void split_usermask(const char *mask, char **nick, char **user, char **host)
{
    char *mask2 = sstrdup(mask);

    *nick = strtok(mask2, "!");
    *user = strtok(NULL, "@");
    *host = strtok(NULL, "");
    /* Handle special case: mask == user@host */
    if (*nick && !*user && strchr(*nick, '@')) {
	*nick = NULL;
	*user = strtok(mask2, "@");
	*host = strtok(NULL, "");
    }
    if (!*nick)
	*nick = "*";
    if (!*user)
	*user = "*";
    if (!*host)
	*host = "*";
    *nick = sstrdup(*nick);
    *user = sstrdup(*user);
    *host = sstrdup(*host);
    free(mask2);
}

/*************************************************************************/

/* Given a user, return a mask that will most likely match any address the
 * user will have from that location.  For IP addresses, wildcards the
 * appropriate subnet mask (e.g. 35.1.1.1 -> 35.*; 128.2.1.1 -> 128.2.*);
 * for named addresses, wildcards the leftmost part of the name unless the
 * name only contains two parts.  If the username begins with a ~, delete
 * it.  The returned character string is malloc'd and should be free'd
 * when done with.
 */

char *create_mask(User *u)
{
    char *mask, *s, *end;

    /* Get us a buffer the size of the username plus hostname.  The result
     * will never be longer than this (and will often be shorter).
     */
    end = mask = smalloc(strlen(u->username) + strlen(u->host) + 2);
    end += sprintf(end, "%s@", u->username);
    if (strspn(u->host, "0123456789.") == strlen(u->host)
		&& (s = strchr(u->host, '.'))
		&& (s = strchr(s+1, '.'))
		&& (s = strchr(s+1, '.'))
		&& (   !strchr(s+1, '.'))) {	/* IP addr */
	s = sstrdup(u->host);
	*strrchr(s, '.') = 0;
	if (atoi(u->host) < 192)
	    *strrchr(s, '.') = 0;
	if (atoi(u->host) < 128)
	    *strrchr(s, '.') = 0;
	sprintf(end, "%s.*", s);
	free(s);
    } else {
	if ((s = strchr(u->host, '.')) && strchr(s+1, '.')) {
	    s = sstrdup(strchr(u->host, '.')-1);
	    *s = '*';
	} else {
	    s = sstrdup(u->host);
	}
	strcpy(end, s);
	free(s);
    }
    return mask;
}
/*********************************************************************/
char *bcreate_mask(User *u,int btype)
{
    char *mask, *s, *end;

    end = mask = smalloc(strlen(u->username) + strlen(u->host) + 2);
    if (btype == 1)
	    end += sprintf(end, "%s@", u->username);
    else
	    end += sprintf(end, "*@");

    if (strspn(u->host, "0123456789.") == strlen(u->host)
		&& (s = strchr(u->host, '.'))
		&& (s = strchr(s+1, '.'))
		&& (s = strchr(s+1, '.'))
		&& (   !strchr(s+1, '.'))) {	/* IP addr */
	s = sstrdup(u->host);
	*strrchr(s, '.') = 0;
	if (atoi(u->host) < 192)
	    *strrchr(s, '.') = 0;
	if (atoi(u->host) < 128)
	    *strrchr(s, '.') = 0;
	sprintf(end, "%s.*", s);
	free(s);
    } else {
	if ((s = strchr(u->host, '.')) && strchr(s+1, '.')) {
	    s = sstrdup(strchr(u->host, '.')-1);
	    *s = '*';
	} else {
	    s = sstrdup(u->host);
	}

      strcpy(end, s);
      free(s);
    }
    return mask;
}
/*********************************************************************/

void mass_check_akill()
{
    User *user;
    for (user = userlist; user; user = user->next) {
       if (check_akill(user->nick,user->username,user->realhost))
          return;           
    }
}

void trace_host(const char *source)
{
    User *user;
    char *host = strtok(NULL, "");
    int i = 0;

    if (!host) {
       notice(s_OperServ, source, "Syntax: \2TRACE\2 HOST");
       notice(s_OperServ, source, "Example: \2TRACE\2 *pleaselook.com");

    } else {
    for (user = userlist; user; user = user->next) {
       if(match_wild_nocase(host, user->host)) {
		++i;
                notice(s_OperServ,source,"%d - %s (%s@%s)", i, user->nick, user->username,user->host);
	}	
    }
    notice(s_OperServ, source, NS_NFEOL, i);
    }
}

int check_akill_num(char *mask)
{
    User *user;
    int i=0;
    char *nick = strtok(mask, "!");
    char *ident = strtok(mask, "@");
    char *host = strtok(mask, "");

    for (user = userlist; user; user = user->next) {
        if (match_wild_nocase(ident, user->username)
          && match_wild_nocase(host, user->realhost))
             i += 1;

    }
  free(mask);
  return i;
}


/*************************************************************************/

int check_ignore_num(const char *mask)
{
    User *user;
    int i=0;
    char *mask2 = sstrdup(mask);
    char *nick = strtok(mask2, "!");
    char *ident2 = strtok(NULL, "!");
    char *ident = strtok(ident2, "@");
    char *host = strtok(NULL, "");

    for (user = userlist; user; user = user->next) {
        if (match_wild_nocase(nick, user->nick)
          && match_wild_nocase(ident, user->username)
          && match_wild_nocase(host, user->host))
             i += 1;
    }
    free(mask2);
//  if (i > 0)
//    i /= 2;
  return i;
}

/**********************************************************************/
void alljoin(char *chan)
{
    User *user;
        for (user = userlist; user; user = user->next) {
        	send_cmd(s_MassServ, "SAJOIN %s %s",user->nick,chan);
		notice(s_MassServ, user->nick, "You are invited to join channel %s by Network Admin",chan);
	}
}
